# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from ... import _utilities
from . import outputs
from ... import core as _core
from ... import meta as _meta

__all__ = [
    'AllocatedDeviceStatus',
    'AllocatedDeviceStatusPatch',
    'AllocationResult',
    'AllocationResultPatch',
    'BasicDevice',
    'BasicDevicePatch',
    'CELDeviceSelector',
    'CELDeviceSelectorPatch',
    'CapacityRequestPolicy',
    'CapacityRequestPolicyRange',
    'CapacityRequirements',
    'CapacityRequirementsPatch',
    'Counter',
    'CounterSet',
    'CounterSetPatch',
    'Device',
    'DeviceAllocationConfiguration',
    'DeviceAllocationConfigurationPatch',
    'DeviceAllocationResult',
    'DeviceAllocationResultPatch',
    'DeviceAttribute',
    'DeviceCapacity',
    'DeviceClaim',
    'DeviceClaimConfiguration',
    'DeviceClaimConfigurationPatch',
    'DeviceClaimPatch',
    'DeviceClass',
    'DeviceClassConfiguration',
    'DeviceClassConfigurationPatch',
    'DeviceClassSpec',
    'DeviceClassSpecPatch',
    'DeviceConstraint',
    'DeviceConstraintPatch',
    'DeviceCounterConsumption',
    'DeviceCounterConsumptionPatch',
    'DevicePatch',
    'DeviceRequest',
    'DeviceRequestAllocationResult',
    'DeviceRequestAllocationResultPatch',
    'DeviceRequestPatch',
    'DeviceSelector',
    'DeviceSelectorPatch',
    'DeviceSubRequest',
    'DeviceSubRequestPatch',
    'DeviceTaint',
    'DeviceTaintPatch',
    'DeviceToleration',
    'DeviceTolerationPatch',
    'NetworkDeviceData',
    'NetworkDeviceDataPatch',
    'OpaqueDeviceConfiguration',
    'OpaqueDeviceConfigurationPatch',
    'ResourceClaim',
    'ResourceClaimConsumerReference',
    'ResourceClaimConsumerReferencePatch',
    'ResourceClaimSpec',
    'ResourceClaimSpecPatch',
    'ResourceClaimStatus',
    'ResourceClaimStatusPatch',
    'ResourceClaimTemplate',
    'ResourceClaimTemplateSpec',
    'ResourceClaimTemplateSpecPatch',
    'ResourcePool',
    'ResourcePoolPatch',
    'ResourceSlice',
    'ResourceSliceSpec',
    'ResourceSliceSpecPatch',
]

@pulumi.output_type
class AllocatedDeviceStatus(dict):
    """
    AllocatedDeviceStatus contains the status of an allocated device, if the driver chooses to report it. This may include driver-specific information.

    The combination of Driver, Pool, Device, and ShareID must match the corresponding key in Status.Allocation.Devices.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "networkData":
            suggest = "network_data"
        elif key == "shareID":
            suggest = "share_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in AllocatedDeviceStatus. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        AllocatedDeviceStatus.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        AllocatedDeviceStatus.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 device: _builtins.str,
                 driver: _builtins.str,
                 pool: _builtins.str,
                 conditions: Optional[Sequence['_meta.v1.outputs.Condition']] = None,
                 data: Optional[Any] = None,
                 network_data: Optional['outputs.NetworkDeviceData'] = None,
                 share_id: Optional[_builtins.str] = None):
        """
        AllocatedDeviceStatus contains the status of an allocated device, if the driver chooses to report it. This may include driver-specific information.

        The combination of Driver, Pool, Device, and ShareID must match the corresponding key in Status.Allocation.Devices.
        :param _builtins.str device: Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        :param _builtins.str driver: Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param _builtins.str pool: This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).
               
               Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        :param Sequence['_meta.v1.ConditionArgs'] conditions: Conditions contains the latest observation of the device's state. If the device has been configured according to the class and claim config references, the `Ready` condition should be True.
               
               Must not contain more than 8 entries.
        :param Any data: Data contains arbitrary driver-specific data.
               
               The length of the raw data must be smaller or equal to 10 Ki.
        :param 'NetworkDeviceDataArgs' network_data: NetworkData contains network-related information specific to the device.
        :param _builtins.str share_id: ShareID uniquely identifies an individual allocation share of the device.
        """
        pulumi.set(__self__, "device", device)
        pulumi.set(__self__, "driver", driver)
        pulumi.set(__self__, "pool", pool)
        if conditions is not None:
            pulumi.set(__self__, "conditions", conditions)
        if data is not None:
            pulumi.set(__self__, "data", data)
        if network_data is not None:
            pulumi.set(__self__, "network_data", network_data)
        if share_id is not None:
            pulumi.set(__self__, "share_id", share_id)

    @_builtins.property
    @pulumi.getter
    def device(self) -> _builtins.str:
        """
        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        """
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter
    def driver(self) -> _builtins.str:
        """
        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> _builtins.str:
        """
        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).

        Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter
    def conditions(self) -> Optional[Sequence['_meta.v1.outputs.Condition']]:
        """
        Conditions contains the latest observation of the device's state. If the device has been configured according to the class and claim config references, the `Ready` condition should be True.

        Must not contain more than 8 entries.
        """
        return pulumi.get(self, "conditions")

    @_builtins.property
    @pulumi.getter
    def data(self) -> Optional[Any]:
        """
        Data contains arbitrary driver-specific data.

        The length of the raw data must be smaller or equal to 10 Ki.
        """
        return pulumi.get(self, "data")

    @_builtins.property
    @pulumi.getter(name="networkData")
    def network_data(self) -> Optional['outputs.NetworkDeviceData']:
        """
        NetworkData contains network-related information specific to the device.
        """
        return pulumi.get(self, "network_data")

    @_builtins.property
    @pulumi.getter(name="shareID")
    def share_id(self) -> Optional[_builtins.str]:
        """
        ShareID uniquely identifies an individual allocation share of the device.
        """
        return pulumi.get(self, "share_id")


@pulumi.output_type
class AllocatedDeviceStatusPatch(dict):
    """
    AllocatedDeviceStatus contains the status of an allocated device, if the driver chooses to report it. This may include driver-specific information.

    The combination of Driver, Pool, Device, and ShareID must match the corresponding key in Status.Allocation.Devices.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "networkData":
            suggest = "network_data"
        elif key == "shareID":
            suggest = "share_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in AllocatedDeviceStatusPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        AllocatedDeviceStatusPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        AllocatedDeviceStatusPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 conditions: Optional[Sequence['_meta.v1.outputs.ConditionPatch']] = None,
                 data: Optional[Any] = None,
                 device: Optional[_builtins.str] = None,
                 driver: Optional[_builtins.str] = None,
                 network_data: Optional['outputs.NetworkDeviceDataPatch'] = None,
                 pool: Optional[_builtins.str] = None,
                 share_id: Optional[_builtins.str] = None):
        """
        AllocatedDeviceStatus contains the status of an allocated device, if the driver chooses to report it. This may include driver-specific information.

        The combination of Driver, Pool, Device, and ShareID must match the corresponding key in Status.Allocation.Devices.
        :param Sequence['_meta.v1.ConditionPatchArgs'] conditions: Conditions contains the latest observation of the device's state. If the device has been configured according to the class and claim config references, the `Ready` condition should be True.
               
               Must not contain more than 8 entries.
        :param Any data: Data contains arbitrary driver-specific data.
               
               The length of the raw data must be smaller or equal to 10 Ki.
        :param _builtins.str device: Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        :param _builtins.str driver: Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param 'NetworkDeviceDataPatchArgs' network_data: NetworkData contains network-related information specific to the device.
        :param _builtins.str pool: This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).
               
               Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        :param _builtins.str share_id: ShareID uniquely identifies an individual allocation share of the device.
        """
        if conditions is not None:
            pulumi.set(__self__, "conditions", conditions)
        if data is not None:
            pulumi.set(__self__, "data", data)
        if device is not None:
            pulumi.set(__self__, "device", device)
        if driver is not None:
            pulumi.set(__self__, "driver", driver)
        if network_data is not None:
            pulumi.set(__self__, "network_data", network_data)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if share_id is not None:
            pulumi.set(__self__, "share_id", share_id)

    @_builtins.property
    @pulumi.getter
    def conditions(self) -> Optional[Sequence['_meta.v1.outputs.ConditionPatch']]:
        """
        Conditions contains the latest observation of the device's state. If the device has been configured according to the class and claim config references, the `Ready` condition should be True.

        Must not contain more than 8 entries.
        """
        return pulumi.get(self, "conditions")

    @_builtins.property
    @pulumi.getter
    def data(self) -> Optional[Any]:
        """
        Data contains arbitrary driver-specific data.

        The length of the raw data must be smaller or equal to 10 Ki.
        """
        return pulumi.get(self, "data")

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[_builtins.str]:
        """
        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        """
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter
    def driver(self) -> Optional[_builtins.str]:
        """
        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter(name="networkData")
    def network_data(self) -> Optional['outputs.NetworkDeviceDataPatch']:
        """
        NetworkData contains network-related information specific to the device.
        """
        return pulumi.get(self, "network_data")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> Optional[_builtins.str]:
        """
        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).

        Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter(name="shareID")
    def share_id(self) -> Optional[_builtins.str]:
        """
        ShareID uniquely identifies an individual allocation share of the device.
        """
        return pulumi.get(self, "share_id")


@pulumi.output_type
class AllocationResult(dict):
    """
    AllocationResult contains attributes of an allocated resource.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allocationTimestamp":
            suggest = "allocation_timestamp"
        elif key == "nodeSelector":
            suggest = "node_selector"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in AllocationResult. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        AllocationResult.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        AllocationResult.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 allocation_timestamp: Optional[_builtins.str] = None,
                 devices: Optional['outputs.DeviceAllocationResult'] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelector'] = None):
        """
        AllocationResult contains attributes of an allocated resource.
        :param _builtins.str allocation_timestamp: AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.
        :param 'DeviceAllocationResultArgs' devices: Devices is the result of allocating devices.
        :param '_core.v1.NodeSelectorArgs' node_selector: NodeSelector defines where the allocated resources are available. If unset, they are available everywhere.
        """
        if allocation_timestamp is not None:
            pulumi.set(__self__, "allocation_timestamp", allocation_timestamp)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)

    @_builtins.property
    @pulumi.getter(name="allocationTimestamp")
    def allocation_timestamp(self) -> Optional[_builtins.str]:
        """
        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.
        """
        return pulumi.get(self, "allocation_timestamp")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional['outputs.DeviceAllocationResult']:
        """
        Devices is the result of allocating devices.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelector']:
        """
        NodeSelector defines where the allocated resources are available. If unset, they are available everywhere.
        """
        return pulumi.get(self, "node_selector")


@pulumi.output_type
class AllocationResultPatch(dict):
    """
    AllocationResult contains attributes of an allocated resource.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allocationTimestamp":
            suggest = "allocation_timestamp"
        elif key == "nodeSelector":
            suggest = "node_selector"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in AllocationResultPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        AllocationResultPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        AllocationResultPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 allocation_timestamp: Optional[_builtins.str] = None,
                 devices: Optional['outputs.DeviceAllocationResultPatch'] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelectorPatch'] = None):
        """
        AllocationResult contains attributes of an allocated resource.
        :param _builtins.str allocation_timestamp: AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.
        :param 'DeviceAllocationResultPatchArgs' devices: Devices is the result of allocating devices.
        :param '_core.v1.NodeSelectorPatchArgs' node_selector: NodeSelector defines where the allocated resources are available. If unset, they are available everywhere.
        """
        if allocation_timestamp is not None:
            pulumi.set(__self__, "allocation_timestamp", allocation_timestamp)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)

    @_builtins.property
    @pulumi.getter(name="allocationTimestamp")
    def allocation_timestamp(self) -> Optional[_builtins.str]:
        """
        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.
        """
        return pulumi.get(self, "allocation_timestamp")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional['outputs.DeviceAllocationResultPatch']:
        """
        Devices is the result of allocating devices.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelectorPatch']:
        """
        NodeSelector defines where the allocated resources are available. If unset, they are available everywhere.
        """
        return pulumi.get(self, "node_selector")


@pulumi.output_type
class BasicDevice(dict):
    """
    BasicDevice defines one device instance.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allNodes":
            suggest = "all_nodes"
        elif key == "allowMultipleAllocations":
            suggest = "allow_multiple_allocations"
        elif key == "bindingConditions":
            suggest = "binding_conditions"
        elif key == "bindingFailureConditions":
            suggest = "binding_failure_conditions"
        elif key == "bindsToNode":
            suggest = "binds_to_node"
        elif key == "consumesCounters":
            suggest = "consumes_counters"
        elif key == "nodeName":
            suggest = "node_name"
        elif key == "nodeSelector":
            suggest = "node_selector"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in BasicDevice. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        BasicDevice.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        BasicDevice.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 all_nodes: Optional[_builtins.bool] = None,
                 allow_multiple_allocations: Optional[_builtins.bool] = None,
                 attributes: Optional[Mapping[str, 'outputs.DeviceAttribute']] = None,
                 binding_conditions: Optional[Sequence[_builtins.str]] = None,
                 binding_failure_conditions: Optional[Sequence[_builtins.str]] = None,
                 binds_to_node: Optional[_builtins.bool] = None,
                 capacity: Optional[Mapping[str, 'outputs.DeviceCapacity']] = None,
                 consumes_counters: Optional[Sequence['outputs.DeviceCounterConsumption']] = None,
                 node_name: Optional[_builtins.str] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelector'] = None,
                 taints: Optional[Sequence['outputs.DeviceTaint']] = None):
        """
        BasicDevice defines one device instance.
        :param _builtins.bool all_nodes: AllNodes indicates that all nodes have access to the device.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param _builtins.bool allow_multiple_allocations: AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.
               
               If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.
        :param Mapping[str, 'DeviceAttributeArgs'] attributes: Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.
               
               The maximum number of attributes and capacities combined is 32.
        :param Sequence[_builtins.str] binding_conditions: BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.
               
               The maximum number of binding conditions is 4.
               
               The conditions must be a valid condition type string.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Sequence[_builtins.str] binding_failure_conditions: BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.
               
               The maximum number of binding failure conditions is 4.
               
               The conditions must be a valid condition type string.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param _builtins.bool binds_to_node: BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Mapping[str, 'DeviceCapacityArgs'] capacity: Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.
               
               The maximum number of attributes and capacities combined is 32.
        :param Sequence['DeviceCounterConsumptionArgs'] consumes_counters: ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.
               
               There can only be a single entry per counterSet.
               
               The maximum number of device counter consumptions per device is 2.
        :param _builtins.str node_name: NodeName identifies the node where the device is available.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param '_core.v1.NodeSelectorArgs' node_selector: NodeSelector defines the nodes where the device is available.
               
               Must use exactly one term.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param Sequence['DeviceTaintArgs'] taints: If specified, these are the driver-defined taints.
               
               The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        if all_nodes is not None:
            pulumi.set(__self__, "all_nodes", all_nodes)
        if allow_multiple_allocations is not None:
            pulumi.set(__self__, "allow_multiple_allocations", allow_multiple_allocations)
        if attributes is not None:
            pulumi.set(__self__, "attributes", attributes)
        if binding_conditions is not None:
            pulumi.set(__self__, "binding_conditions", binding_conditions)
        if binding_failure_conditions is not None:
            pulumi.set(__self__, "binding_failure_conditions", binding_failure_conditions)
        if binds_to_node is not None:
            pulumi.set(__self__, "binds_to_node", binds_to_node)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if consumes_counters is not None:
            pulumi.set(__self__, "consumes_counters", consumes_counters)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)
        if taints is not None:
            pulumi.set(__self__, "taints", taints)

    @_builtins.property
    @pulumi.getter(name="allNodes")
    def all_nodes(self) -> Optional[_builtins.bool]:
        """
        AllNodes indicates that all nodes have access to the device.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "all_nodes")

    @_builtins.property
    @pulumi.getter(name="allowMultipleAllocations")
    def allow_multiple_allocations(self) -> Optional[_builtins.bool]:
        """
        AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.

        If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.
        """
        return pulumi.get(self, "allow_multiple_allocations")

    @_builtins.property
    @pulumi.getter
    def attributes(self) -> Optional[Mapping[str, 'outputs.DeviceAttribute']]:
        """
        Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.

        The maximum number of attributes and capacities combined is 32.
        """
        return pulumi.get(self, "attributes")

    @_builtins.property
    @pulumi.getter(name="bindingConditions")
    def binding_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.

        The maximum number of binding conditions is 4.

        The conditions must be a valid condition type string.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_conditions")

    @_builtins.property
    @pulumi.getter(name="bindingFailureConditions")
    def binding_failure_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.

        The maximum number of binding failure conditions is 4.

        The conditions must be a valid condition type string.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_failure_conditions")

    @_builtins.property
    @pulumi.getter(name="bindsToNode")
    def binds_to_node(self) -> Optional[_builtins.bool]:
        """
        BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binds_to_node")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional[Mapping[str, 'outputs.DeviceCapacity']]:
        """
        Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.

        The maximum number of attributes and capacities combined is 32.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter(name="consumesCounters")
    def consumes_counters(self) -> Optional[Sequence['outputs.DeviceCounterConsumption']]:
        """
        ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.

        There can only be a single entry per counterSet.

        The maximum number of device counter consumptions per device is 2.
        """
        return pulumi.get(self, "consumes_counters")

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[_builtins.str]:
        """
        NodeName identifies the node where the device is available.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "node_name")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelector']:
        """
        NodeSelector defines the nodes where the device is available.

        Must use exactly one term.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "node_selector")

    @_builtins.property
    @pulumi.getter
    def taints(self) -> Optional[Sequence['outputs.DeviceTaint']]:
        """
        If specified, these are the driver-defined taints.

        The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "taints")


@pulumi.output_type
class BasicDevicePatch(dict):
    """
    BasicDevice defines one device instance.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allNodes":
            suggest = "all_nodes"
        elif key == "allowMultipleAllocations":
            suggest = "allow_multiple_allocations"
        elif key == "bindingConditions":
            suggest = "binding_conditions"
        elif key == "bindingFailureConditions":
            suggest = "binding_failure_conditions"
        elif key == "bindsToNode":
            suggest = "binds_to_node"
        elif key == "consumesCounters":
            suggest = "consumes_counters"
        elif key == "nodeName":
            suggest = "node_name"
        elif key == "nodeSelector":
            suggest = "node_selector"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in BasicDevicePatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        BasicDevicePatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        BasicDevicePatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 all_nodes: Optional[_builtins.bool] = None,
                 allow_multiple_allocations: Optional[_builtins.bool] = None,
                 attributes: Optional[Mapping[str, 'outputs.DeviceAttribute']] = None,
                 binding_conditions: Optional[Sequence[_builtins.str]] = None,
                 binding_failure_conditions: Optional[Sequence[_builtins.str]] = None,
                 binds_to_node: Optional[_builtins.bool] = None,
                 capacity: Optional[Mapping[str, 'outputs.DeviceCapacity']] = None,
                 consumes_counters: Optional[Sequence['outputs.DeviceCounterConsumptionPatch']] = None,
                 node_name: Optional[_builtins.str] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelectorPatch'] = None,
                 taints: Optional[Sequence['outputs.DeviceTaintPatch']] = None):
        """
        BasicDevice defines one device instance.
        :param _builtins.bool all_nodes: AllNodes indicates that all nodes have access to the device.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param _builtins.bool allow_multiple_allocations: AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.
               
               If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.
        :param Mapping[str, 'DeviceAttributeArgs'] attributes: Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.
               
               The maximum number of attributes and capacities combined is 32.
        :param Sequence[_builtins.str] binding_conditions: BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.
               
               The maximum number of binding conditions is 4.
               
               The conditions must be a valid condition type string.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Sequence[_builtins.str] binding_failure_conditions: BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.
               
               The maximum number of binding failure conditions is 4.
               
               The conditions must be a valid condition type string.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param _builtins.bool binds_to_node: BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Mapping[str, 'DeviceCapacityArgs'] capacity: Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.
               
               The maximum number of attributes and capacities combined is 32.
        :param Sequence['DeviceCounterConsumptionPatchArgs'] consumes_counters: ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.
               
               There can only be a single entry per counterSet.
               
               The maximum number of device counter consumptions per device is 2.
        :param _builtins.str node_name: NodeName identifies the node where the device is available.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param '_core.v1.NodeSelectorPatchArgs' node_selector: NodeSelector defines the nodes where the device is available.
               
               Must use exactly one term.
               
               Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        :param Sequence['DeviceTaintPatchArgs'] taints: If specified, these are the driver-defined taints.
               
               The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        if all_nodes is not None:
            pulumi.set(__self__, "all_nodes", all_nodes)
        if allow_multiple_allocations is not None:
            pulumi.set(__self__, "allow_multiple_allocations", allow_multiple_allocations)
        if attributes is not None:
            pulumi.set(__self__, "attributes", attributes)
        if binding_conditions is not None:
            pulumi.set(__self__, "binding_conditions", binding_conditions)
        if binding_failure_conditions is not None:
            pulumi.set(__self__, "binding_failure_conditions", binding_failure_conditions)
        if binds_to_node is not None:
            pulumi.set(__self__, "binds_to_node", binds_to_node)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if consumes_counters is not None:
            pulumi.set(__self__, "consumes_counters", consumes_counters)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)
        if taints is not None:
            pulumi.set(__self__, "taints", taints)

    @_builtins.property
    @pulumi.getter(name="allNodes")
    def all_nodes(self) -> Optional[_builtins.bool]:
        """
        AllNodes indicates that all nodes have access to the device.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "all_nodes")

    @_builtins.property
    @pulumi.getter(name="allowMultipleAllocations")
    def allow_multiple_allocations(self) -> Optional[_builtins.bool]:
        """
        AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.

        If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.
        """
        return pulumi.get(self, "allow_multiple_allocations")

    @_builtins.property
    @pulumi.getter
    def attributes(self) -> Optional[Mapping[str, 'outputs.DeviceAttribute']]:
        """
        Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.

        The maximum number of attributes and capacities combined is 32.
        """
        return pulumi.get(self, "attributes")

    @_builtins.property
    @pulumi.getter(name="bindingConditions")
    def binding_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.

        The maximum number of binding conditions is 4.

        The conditions must be a valid condition type string.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_conditions")

    @_builtins.property
    @pulumi.getter(name="bindingFailureConditions")
    def binding_failure_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.

        The maximum number of binding failure conditions is 4.

        The conditions must be a valid condition type string.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_failure_conditions")

    @_builtins.property
    @pulumi.getter(name="bindsToNode")
    def binds_to_node(self) -> Optional[_builtins.bool]:
        """
        BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binds_to_node")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional[Mapping[str, 'outputs.DeviceCapacity']]:
        """
        Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.

        The maximum number of attributes and capacities combined is 32.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter(name="consumesCounters")
    def consumes_counters(self) -> Optional[Sequence['outputs.DeviceCounterConsumptionPatch']]:
        """
        ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.

        There can only be a single entry per counterSet.

        The maximum number of device counter consumptions per device is 2.
        """
        return pulumi.get(self, "consumes_counters")

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[_builtins.str]:
        """
        NodeName identifies the node where the device is available.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "node_name")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelectorPatch']:
        """
        NodeSelector defines the nodes where the device is available.

        Must use exactly one term.

        Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.
        """
        return pulumi.get(self, "node_selector")

    @_builtins.property
    @pulumi.getter
    def taints(self) -> Optional[Sequence['outputs.DeviceTaintPatch']]:
        """
        If specified, these are the driver-defined taints.

        The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "taints")


@pulumi.output_type
class CELDeviceSelector(dict):
    """
    CELDeviceSelector contains a CEL expression for selecting a device.
    """
    def __init__(__self__, *,
                 expression: _builtins.str):
        """
        CELDeviceSelector contains a CEL expression for selecting a device.
        :param _builtins.str expression: Expression is a CEL expression which evaluates a single device. It must evaluate to true when the device under consideration satisfies the desired criteria, and false when it does not. Any other result is an error and causes allocation of devices to abort.
               
               The expression's input is an object named "device", which carries the following properties:
                - driver (string): the name of the driver which defines this device.
                - attributes (map[string]object): the device's attributes, grouped by prefix
                  (e.g. device.attributes["dra.example.com"] evaluates to an object with all
                  of the attributes which were prefixed by "dra.example.com".
                - capacity (map[string]object): the device's capacities, grouped by prefix.
                - allowMultipleAllocations (bool): the allowMultipleAllocations property of the device
                  (v1.34+ with the DRAConsumableCapacity feature enabled).
               
               Example: Consider a device with driver="dra.example.com", which exposes two attributes named "model" and "ext.example.com/family" and which exposes one capacity named "modules". This input to this expression would have the following fields:
               
                   device.driver
                   device.attributes["dra.example.com"].model
                   device.attributes["ext.example.com"].family
                   device.capacity["dra.example.com"].modules
               
               The device.driver field can be used to check for a specific driver, either as a high-level precondition (i.e. you only want to consider devices from this driver) or as part of a multi-clause expression that is meant to consider devices from different drivers.
               
               The value type of each attribute is defined by the device definition, and users who write these expressions must consult the documentation for their specific drivers. The value type of each capacity is Quantity.
               
               If an unknown prefix is used as a lookup in either device.attributes or device.capacity, an empty map will be returned. Any reference to an unknown field will cause an evaluation error and allocation to abort.
               
               A robust expression should check for the existence of attributes before referencing them.
               
               For ease of use, the cel.bind() function is enabled, and can be used to simplify expressions that access multiple attributes with the same domain. For example:
               
                   cel.bind(dra, device.attributes["dra.example.com"], dra.someBool && dra.anotherBool)
               
               The length of the expression must be smaller or equal to 10 Ki. The cost of evaluating it is also limited based on the estimated number of logical steps.
        """
        pulumi.set(__self__, "expression", expression)

    @_builtins.property
    @pulumi.getter
    def expression(self) -> _builtins.str:
        """
        Expression is a CEL expression which evaluates a single device. It must evaluate to true when the device under consideration satisfies the desired criteria, and false when it does not. Any other result is an error and causes allocation of devices to abort.

        The expression's input is an object named "device", which carries the following properties:
         - driver (string): the name of the driver which defines this device.
         - attributes (map[string]object): the device's attributes, grouped by prefix
           (e.g. device.attributes["dra.example.com"] evaluates to an object with all
           of the attributes which were prefixed by "dra.example.com".
         - capacity (map[string]object): the device's capacities, grouped by prefix.
         - allowMultipleAllocations (bool): the allowMultipleAllocations property of the device
           (v1.34+ with the DRAConsumableCapacity feature enabled).

        Example: Consider a device with driver="dra.example.com", which exposes two attributes named "model" and "ext.example.com/family" and which exposes one capacity named "modules". This input to this expression would have the following fields:

            device.driver
            device.attributes["dra.example.com"].model
            device.attributes["ext.example.com"].family
            device.capacity["dra.example.com"].modules

        The device.driver field can be used to check for a specific driver, either as a high-level precondition (i.e. you only want to consider devices from this driver) or as part of a multi-clause expression that is meant to consider devices from different drivers.

        The value type of each attribute is defined by the device definition, and users who write these expressions must consult the documentation for their specific drivers. The value type of each capacity is Quantity.

        If an unknown prefix is used as a lookup in either device.attributes or device.capacity, an empty map will be returned. Any reference to an unknown field will cause an evaluation error and allocation to abort.

        A robust expression should check for the existence of attributes before referencing them.

        For ease of use, the cel.bind() function is enabled, and can be used to simplify expressions that access multiple attributes with the same domain. For example:

            cel.bind(dra, device.attributes["dra.example.com"], dra.someBool && dra.anotherBool)

        The length of the expression must be smaller or equal to 10 Ki. The cost of evaluating it is also limited based on the estimated number of logical steps.
        """
        return pulumi.get(self, "expression")


@pulumi.output_type
class CELDeviceSelectorPatch(dict):
    """
    CELDeviceSelector contains a CEL expression for selecting a device.
    """
    def __init__(__self__, *,
                 expression: Optional[_builtins.str] = None):
        """
        CELDeviceSelector contains a CEL expression for selecting a device.
        :param _builtins.str expression: Expression is a CEL expression which evaluates a single device. It must evaluate to true when the device under consideration satisfies the desired criteria, and false when it does not. Any other result is an error and causes allocation of devices to abort.
               
               The expression's input is an object named "device", which carries the following properties:
                - driver (string): the name of the driver which defines this device.
                - attributes (map[string]object): the device's attributes, grouped by prefix
                  (e.g. device.attributes["dra.example.com"] evaluates to an object with all
                  of the attributes which were prefixed by "dra.example.com".
                - capacity (map[string]object): the device's capacities, grouped by prefix.
                - allowMultipleAllocations (bool): the allowMultipleAllocations property of the device
                  (v1.34+ with the DRAConsumableCapacity feature enabled).
               
               Example: Consider a device with driver="dra.example.com", which exposes two attributes named "model" and "ext.example.com/family" and which exposes one capacity named "modules". This input to this expression would have the following fields:
               
                   device.driver
                   device.attributes["dra.example.com"].model
                   device.attributes["ext.example.com"].family
                   device.capacity["dra.example.com"].modules
               
               The device.driver field can be used to check for a specific driver, either as a high-level precondition (i.e. you only want to consider devices from this driver) or as part of a multi-clause expression that is meant to consider devices from different drivers.
               
               The value type of each attribute is defined by the device definition, and users who write these expressions must consult the documentation for their specific drivers. The value type of each capacity is Quantity.
               
               If an unknown prefix is used as a lookup in either device.attributes or device.capacity, an empty map will be returned. Any reference to an unknown field will cause an evaluation error and allocation to abort.
               
               A robust expression should check for the existence of attributes before referencing them.
               
               For ease of use, the cel.bind() function is enabled, and can be used to simplify expressions that access multiple attributes with the same domain. For example:
               
                   cel.bind(dra, device.attributes["dra.example.com"], dra.someBool && dra.anotherBool)
               
               The length of the expression must be smaller or equal to 10 Ki. The cost of evaluating it is also limited based on the estimated number of logical steps.
        """
        if expression is not None:
            pulumi.set(__self__, "expression", expression)

    @_builtins.property
    @pulumi.getter
    def expression(self) -> Optional[_builtins.str]:
        """
        Expression is a CEL expression which evaluates a single device. It must evaluate to true when the device under consideration satisfies the desired criteria, and false when it does not. Any other result is an error and causes allocation of devices to abort.

        The expression's input is an object named "device", which carries the following properties:
         - driver (string): the name of the driver which defines this device.
         - attributes (map[string]object): the device's attributes, grouped by prefix
           (e.g. device.attributes["dra.example.com"] evaluates to an object with all
           of the attributes which were prefixed by "dra.example.com".
         - capacity (map[string]object): the device's capacities, grouped by prefix.
         - allowMultipleAllocations (bool): the allowMultipleAllocations property of the device
           (v1.34+ with the DRAConsumableCapacity feature enabled).

        Example: Consider a device with driver="dra.example.com", which exposes two attributes named "model" and "ext.example.com/family" and which exposes one capacity named "modules". This input to this expression would have the following fields:

            device.driver
            device.attributes["dra.example.com"].model
            device.attributes["ext.example.com"].family
            device.capacity["dra.example.com"].modules

        The device.driver field can be used to check for a specific driver, either as a high-level precondition (i.e. you only want to consider devices from this driver) or as part of a multi-clause expression that is meant to consider devices from different drivers.

        The value type of each attribute is defined by the device definition, and users who write these expressions must consult the documentation for their specific drivers. The value type of each capacity is Quantity.

        If an unknown prefix is used as a lookup in either device.attributes or device.capacity, an empty map will be returned. Any reference to an unknown field will cause an evaluation error and allocation to abort.

        A robust expression should check for the existence of attributes before referencing them.

        For ease of use, the cel.bind() function is enabled, and can be used to simplify expressions that access multiple attributes with the same domain. For example:

            cel.bind(dra, device.attributes["dra.example.com"], dra.someBool && dra.anotherBool)

        The length of the expression must be smaller or equal to 10 Ki. The cost of evaluating it is also limited based on the estimated number of logical steps.
        """
        return pulumi.get(self, "expression")


@pulumi.output_type
class CapacityRequestPolicy(dict):
    """
    CapacityRequestPolicy defines how requests consume device capacity.

    Must not set more than one ValidRequestValues.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "validRange":
            suggest = "valid_range"
        elif key == "validValues":
            suggest = "valid_values"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CapacityRequestPolicy. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CapacityRequestPolicy.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CapacityRequestPolicy.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 default: Optional[_builtins.str] = None,
                 valid_range: Optional['outputs.CapacityRequestPolicyRange'] = None,
                 valid_values: Optional[Sequence[_builtins.str]] = None):
        """
        CapacityRequestPolicy defines how requests consume device capacity.

        Must not set more than one ValidRequestValues.
        :param _builtins.str default: Default specifies how much of this capacity is consumed by a request that does not contain an entry for it in DeviceRequest's Capacity.
        :param 'CapacityRequestPolicyRangeArgs' valid_range: ValidRange defines an acceptable quantity value range in consuming requests.
               
               If this field is set, Default must be defined and it must fall within the defined ValidRange.
               
               If the requested amount does not fall within the defined range, the request violates the policy, and this device cannot be allocated.
               
               If the request doesn't contain this capacity entry, Default value is used.
        :param Sequence[_builtins.str] valid_values: ValidValues defines a set of acceptable quantity values in consuming requests.
               
               Must not contain more than 10 entries. Must be sorted in ascending order.
               
               If this field is set, Default must be defined and it must be included in ValidValues list.
               
               If the requested amount does not match any valid value but smaller than some valid values, the scheduler calculates the smallest valid value that is greater than or equal to the request. That is: min(ceil(requestedValue) ∈ validValues), where requestedValue ≤ max(validValues).
               
               If the requested amount exceeds all valid values, the request violates the policy, and this device cannot be allocated.
        """
        if default is not None:
            pulumi.set(__self__, "default", default)
        if valid_range is not None:
            pulumi.set(__self__, "valid_range", valid_range)
        if valid_values is not None:
            pulumi.set(__self__, "valid_values", valid_values)

    @_builtins.property
    @pulumi.getter
    def default(self) -> Optional[_builtins.str]:
        """
        Default specifies how much of this capacity is consumed by a request that does not contain an entry for it in DeviceRequest's Capacity.
        """
        return pulumi.get(self, "default")

    @_builtins.property
    @pulumi.getter(name="validRange")
    def valid_range(self) -> Optional['outputs.CapacityRequestPolicyRange']:
        """
        ValidRange defines an acceptable quantity value range in consuming requests.

        If this field is set, Default must be defined and it must fall within the defined ValidRange.

        If the requested amount does not fall within the defined range, the request violates the policy, and this device cannot be allocated.

        If the request doesn't contain this capacity entry, Default value is used.
        """
        return pulumi.get(self, "valid_range")

    @_builtins.property
    @pulumi.getter(name="validValues")
    def valid_values(self) -> Optional[Sequence[_builtins.str]]:
        """
        ValidValues defines a set of acceptable quantity values in consuming requests.

        Must not contain more than 10 entries. Must be sorted in ascending order.

        If this field is set, Default must be defined and it must be included in ValidValues list.

        If the requested amount does not match any valid value but smaller than some valid values, the scheduler calculates the smallest valid value that is greater than or equal to the request. That is: min(ceil(requestedValue) ∈ validValues), where requestedValue ≤ max(validValues).

        If the requested amount exceeds all valid values, the request violates the policy, and this device cannot be allocated.
        """
        return pulumi.get(self, "valid_values")


@pulumi.output_type
class CapacityRequestPolicyRange(dict):
    """
    CapacityRequestPolicyRange defines a valid range for consumable capacity values.

      - If the requested amount is less than Min, it is rounded up to the Min value.
      - If Step is set and the requested amount is between Min and Max but not aligned with Step,
        it will be rounded up to the next value equal to Min + (n * Step).
      - If Step is not set, the requested amount is used as-is if it falls within the range Min to Max (if set).
      - If the requested or rounded amount exceeds Max (if set), the request does not satisfy the policy,
        and the device cannot be allocated.
    """
    def __init__(__self__, *,
                 min: _builtins.str,
                 max: Optional[_builtins.str] = None,
                 step: Optional[_builtins.str] = None):
        """
        CapacityRequestPolicyRange defines a valid range for consumable capacity values.

          - If the requested amount is less than Min, it is rounded up to the Min value.
          - If Step is set and the requested amount is between Min and Max but not aligned with Step,
            it will be rounded up to the next value equal to Min + (n * Step).
          - If Step is not set, the requested amount is used as-is if it falls within the range Min to Max (if set).
          - If the requested or rounded amount exceeds Max (if set), the request does not satisfy the policy,
            and the device cannot be allocated.
        :param _builtins.str min: Min specifies the minimum capacity allowed for a consumption request.
               
               Min must be greater than or equal to zero, and less than or equal to the capacity value. requestPolicy.default must be more than or equal to the minimum.
        :param _builtins.str max: Max defines the upper limit for capacity that can be requested.
               
               Max must be less than or equal to the capacity value. Min and requestPolicy.default must be less than or equal to the maximum.
        :param _builtins.str step: Step defines the step size between valid capacity amounts within the range.
               
               Max (if set) and requestPolicy.default must be a multiple of Step. Min + Step must be less than or equal to the capacity value.
        """
        pulumi.set(__self__, "min", min)
        if max is not None:
            pulumi.set(__self__, "max", max)
        if step is not None:
            pulumi.set(__self__, "step", step)

    @_builtins.property
    @pulumi.getter
    def min(self) -> _builtins.str:
        """
        Min specifies the minimum capacity allowed for a consumption request.

        Min must be greater than or equal to zero, and less than or equal to the capacity value. requestPolicy.default must be more than or equal to the minimum.
        """
        return pulumi.get(self, "min")

    @_builtins.property
    @pulumi.getter
    def max(self) -> Optional[_builtins.str]:
        """
        Max defines the upper limit for capacity that can be requested.

        Max must be less than or equal to the capacity value. Min and requestPolicy.default must be less than or equal to the maximum.
        """
        return pulumi.get(self, "max")

    @_builtins.property
    @pulumi.getter
    def step(self) -> Optional[_builtins.str]:
        """
        Step defines the step size between valid capacity amounts within the range.

        Max (if set) and requestPolicy.default must be a multiple of Step. Min + Step must be less than or equal to the capacity value.
        """
        return pulumi.get(self, "step")


@pulumi.output_type
class CapacityRequirements(dict):
    """
    CapacityRequirements defines the capacity requirements for a specific device request.
    """
    def __init__(__self__, *,
                 requests: Optional[Mapping[str, _builtins.str]] = None):
        """
        CapacityRequirements defines the capacity requirements for a specific device request.
        :param Mapping[str, _builtins.str] requests: Requests represent individual device resource requests for distinct resources, all of which must be provided by the device.
               
               This value is used as an additional filtering condition against the available capacity on the device. This is semantically equivalent to a CEL selector with `device.capacity[<domain>].<name>.compareTo(quantity(<request quantity>)) >= 0`. For example, device.capacity['test-driver.cdi.k8s.io'].counters.compareTo(quantity('2')) >= 0.
               
               When a requestPolicy is defined, the requested amount is adjusted upward to the nearest valid value based on the policy. If the requested amount cannot be adjusted to a valid value—because it exceeds what the requestPolicy allows— the device is considered ineligible for allocation.
               
               For any capacity that is not explicitly requested: - If no requestPolicy is set, the default consumed capacity is equal to the full device capacity
                 (i.e., the whole device is claimed).
               - If a requestPolicy is set, the default consumed capacity is determined according to that policy.
               
               If the device allows multiple allocation, the aggregated amount across all requests must not exceed the capacity value. The consumed capacity, which may be adjusted based on the requestPolicy if defined, is recorded in the resource claim’s status.devices[*].consumedCapacity field.
        """
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Mapping[str, _builtins.str]]:
        """
        Requests represent individual device resource requests for distinct resources, all of which must be provided by the device.

        This value is used as an additional filtering condition against the available capacity on the device. This is semantically equivalent to a CEL selector with `device.capacity[<domain>].<name>.compareTo(quantity(<request quantity>)) >= 0`. For example, device.capacity['test-driver.cdi.k8s.io'].counters.compareTo(quantity('2')) >= 0.

        When a requestPolicy is defined, the requested amount is adjusted upward to the nearest valid value based on the policy. If the requested amount cannot be adjusted to a valid value—because it exceeds what the requestPolicy allows— the device is considered ineligible for allocation.

        For any capacity that is not explicitly requested: - If no requestPolicy is set, the default consumed capacity is equal to the full device capacity
          (i.e., the whole device is claimed).
        - If a requestPolicy is set, the default consumed capacity is determined according to that policy.

        If the device allows multiple allocation, the aggregated amount across all requests must not exceed the capacity value. The consumed capacity, which may be adjusted based on the requestPolicy if defined, is recorded in the resource claim’s status.devices[*].consumedCapacity field.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class CapacityRequirementsPatch(dict):
    """
    CapacityRequirements defines the capacity requirements for a specific device request.
    """
    def __init__(__self__, *,
                 requests: Optional[Mapping[str, _builtins.str]] = None):
        """
        CapacityRequirements defines the capacity requirements for a specific device request.
        :param Mapping[str, _builtins.str] requests: Requests represent individual device resource requests for distinct resources, all of which must be provided by the device.
               
               This value is used as an additional filtering condition against the available capacity on the device. This is semantically equivalent to a CEL selector with `device.capacity[<domain>].<name>.compareTo(quantity(<request quantity>)) >= 0`. For example, device.capacity['test-driver.cdi.k8s.io'].counters.compareTo(quantity('2')) >= 0.
               
               When a requestPolicy is defined, the requested amount is adjusted upward to the nearest valid value based on the policy. If the requested amount cannot be adjusted to a valid value—because it exceeds what the requestPolicy allows— the device is considered ineligible for allocation.
               
               For any capacity that is not explicitly requested: - If no requestPolicy is set, the default consumed capacity is equal to the full device capacity
                 (i.e., the whole device is claimed).
               - If a requestPolicy is set, the default consumed capacity is determined according to that policy.
               
               If the device allows multiple allocation, the aggregated amount across all requests must not exceed the capacity value. The consumed capacity, which may be adjusted based on the requestPolicy if defined, is recorded in the resource claim’s status.devices[*].consumedCapacity field.
        """
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Mapping[str, _builtins.str]]:
        """
        Requests represent individual device resource requests for distinct resources, all of which must be provided by the device.

        This value is used as an additional filtering condition against the available capacity on the device. This is semantically equivalent to a CEL selector with `device.capacity[<domain>].<name>.compareTo(quantity(<request quantity>)) >= 0`. For example, device.capacity['test-driver.cdi.k8s.io'].counters.compareTo(quantity('2')) >= 0.

        When a requestPolicy is defined, the requested amount is adjusted upward to the nearest valid value based on the policy. If the requested amount cannot be adjusted to a valid value—because it exceeds what the requestPolicy allows— the device is considered ineligible for allocation.

        For any capacity that is not explicitly requested: - If no requestPolicy is set, the default consumed capacity is equal to the full device capacity
          (i.e., the whole device is claimed).
        - If a requestPolicy is set, the default consumed capacity is determined according to that policy.

        If the device allows multiple allocation, the aggregated amount across all requests must not exceed the capacity value. The consumed capacity, which may be adjusted based on the requestPolicy if defined, is recorded in the resource claim’s status.devices[*].consumedCapacity field.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class Counter(dict):
    """
    Counter describes a quantity associated with a device.
    """
    def __init__(__self__, *,
                 value: _builtins.str):
        """
        Counter describes a quantity associated with a device.
        :param _builtins.str value: Value defines how much of a certain device counter is available.
        """
        pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def value(self) -> _builtins.str:
        """
        Value defines how much of a certain device counter is available.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class CounterSet(dict):
    """
    CounterSet defines a named set of counters that are available to be used by devices defined in the ResourcePool.

    The counters are not allocatable by themselves, but can be referenced by devices. When a device is allocated, the portion of counters it uses will no longer be available for use by other devices.
    """
    def __init__(__self__, *,
                 counters: Mapping[str, 'outputs.Counter'],
                 name: _builtins.str):
        """
        CounterSet defines a named set of counters that are available to be used by devices defined in the ResourcePool.

        The counters are not allocatable by themselves, but can be referenced by devices. When a device is allocated, the portion of counters it uses will no longer be available for use by other devices.
        :param Mapping[str, 'CounterArgs'] counters: Counters defines the set of counters for this CounterSet The name of each counter must be unique in that set and must be a DNS label.
               
               The maximum number of counters is 32.
        :param _builtins.str name: Name defines the name of the counter set. It must be a DNS label.
        """
        pulumi.set(__self__, "counters", counters)
        pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter
    def counters(self) -> Mapping[str, 'outputs.Counter']:
        """
        Counters defines the set of counters for this CounterSet The name of each counter must be unique in that set and must be a DNS label.

        The maximum number of counters is 32.
        """
        return pulumi.get(self, "counters")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name defines the name of the counter set. It must be a DNS label.
        """
        return pulumi.get(self, "name")


@pulumi.output_type
class CounterSetPatch(dict):
    """
    CounterSet defines a named set of counters that are available to be used by devices defined in the ResourcePool.

    The counters are not allocatable by themselves, but can be referenced by devices. When a device is allocated, the portion of counters it uses will no longer be available for use by other devices.
    """
    def __init__(__self__, *,
                 counters: Optional[Mapping[str, 'outputs.Counter']] = None,
                 name: Optional[_builtins.str] = None):
        """
        CounterSet defines a named set of counters that are available to be used by devices defined in the ResourcePool.

        The counters are not allocatable by themselves, but can be referenced by devices. When a device is allocated, the portion of counters it uses will no longer be available for use by other devices.
        :param Mapping[str, 'CounterArgs'] counters: Counters defines the set of counters for this CounterSet The name of each counter must be unique in that set and must be a DNS label.
               
               The maximum number of counters is 32.
        :param _builtins.str name: Name defines the name of the counter set. It must be a DNS label.
        """
        if counters is not None:
            pulumi.set(__self__, "counters", counters)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter
    def counters(self) -> Optional[Mapping[str, 'outputs.Counter']]:
        """
        Counters defines the set of counters for this CounterSet The name of each counter must be unique in that set and must be a DNS label.

        The maximum number of counters is 32.
        """
        return pulumi.get(self, "counters")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name defines the name of the counter set. It must be a DNS label.
        """
        return pulumi.get(self, "name")


@pulumi.output_type
class Device(dict):
    """
    Device represents one individual hardware instance that can be selected based on its attributes. Besides the name, exactly one field must be set.
    """
    def __init__(__self__, *,
                 name: _builtins.str,
                 basic: Optional['outputs.BasicDevice'] = None):
        """
        Device represents one individual hardware instance that can be selected based on its attributes. Besides the name, exactly one field must be set.
        :param _builtins.str name: Name is unique identifier among all devices managed by the driver in the pool. It must be a DNS label.
        :param 'BasicDeviceArgs' basic: Basic defines one device instance.
        """
        pulumi.set(__self__, "name", name)
        if basic is not None:
            pulumi.set(__self__, "basic", basic)

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name is unique identifier among all devices managed by the driver in the pool. It must be a DNS label.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def basic(self) -> Optional['outputs.BasicDevice']:
        """
        Basic defines one device instance.
        """
        return pulumi.get(self, "basic")


@pulumi.output_type
class DeviceAllocationConfiguration(dict):
    """
    DeviceAllocationConfiguration gets embedded in an AllocationResult.
    """
    def __init__(__self__, *,
                 source: _builtins.str,
                 opaque: Optional['outputs.OpaqueDeviceConfiguration'] = None,
                 requests: Optional[Sequence[_builtins.str]] = None):
        """
        DeviceAllocationConfiguration gets embedded in an AllocationResult.
        :param _builtins.str source: Source records whether the configuration comes from a class and thus is not something that a normal user would have been able to set or from a claim.
        :param 'OpaqueDeviceConfigurationArgs' opaque: Opaque provides driver-specific configuration parameters.
        :param Sequence[_builtins.str] requests: Requests lists the names of requests where the configuration applies. If empty, its applies to all requests.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        pulumi.set(__self__, "source", source)
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def source(self) -> _builtins.str:
        """
        Source records whether the configuration comes from a class and thus is not something that a normal user would have been able to set or from a claim.
        """
        return pulumi.get(self, "source")

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfiguration']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests lists the names of requests where the configuration applies. If empty, its applies to all requests.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceAllocationConfigurationPatch(dict):
    """
    DeviceAllocationConfiguration gets embedded in an AllocationResult.
    """
    def __init__(__self__, *,
                 opaque: Optional['outputs.OpaqueDeviceConfigurationPatch'] = None,
                 requests: Optional[Sequence[_builtins.str]] = None,
                 source: Optional[_builtins.str] = None):
        """
        DeviceAllocationConfiguration gets embedded in an AllocationResult.
        :param 'OpaqueDeviceConfigurationPatchArgs' opaque: Opaque provides driver-specific configuration parameters.
        :param Sequence[_builtins.str] requests: Requests lists the names of requests where the configuration applies. If empty, its applies to all requests.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        :param _builtins.str source: Source records whether the configuration comes from a class and thus is not something that a normal user would have been able to set or from a claim.
        """
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)
        if source is not None:
            pulumi.set(__self__, "source", source)

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfigurationPatch']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests lists the names of requests where the configuration applies. If empty, its applies to all requests.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        return pulumi.get(self, "requests")

    @_builtins.property
    @pulumi.getter
    def source(self) -> Optional[_builtins.str]:
        """
        Source records whether the configuration comes from a class and thus is not something that a normal user would have been able to set or from a claim.
        """
        return pulumi.get(self, "source")


@pulumi.output_type
class DeviceAllocationResult(dict):
    """
    DeviceAllocationResult is the result of allocating devices.
    """
    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceAllocationConfiguration']] = None,
                 results: Optional[Sequence['outputs.DeviceRequestAllocationResult']] = None):
        """
        DeviceAllocationResult is the result of allocating devices.
        :param Sequence['DeviceAllocationConfigurationArgs'] config: This field is a combination of all the claim and class configuration parameters. Drivers can distinguish between those based on a flag.
               
               This includes configuration parameters for drivers which have no allocated devices in the result because it is up to the drivers which configuration parameters they support. They can silently ignore unknown configuration parameters.
        :param Sequence['DeviceRequestAllocationResultArgs'] results: Results lists all allocated devices.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if results is not None:
            pulumi.set(__self__, "results", results)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceAllocationConfiguration']]:
        """
        This field is a combination of all the claim and class configuration parameters. Drivers can distinguish between those based on a flag.

        This includes configuration parameters for drivers which have no allocated devices in the result because it is up to the drivers which configuration parameters they support. They can silently ignore unknown configuration parameters.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter
    def results(self) -> Optional[Sequence['outputs.DeviceRequestAllocationResult']]:
        """
        Results lists all allocated devices.
        """
        return pulumi.get(self, "results")


@pulumi.output_type
class DeviceAllocationResultPatch(dict):
    """
    DeviceAllocationResult is the result of allocating devices.
    """
    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceAllocationConfigurationPatch']] = None,
                 results: Optional[Sequence['outputs.DeviceRequestAllocationResultPatch']] = None):
        """
        DeviceAllocationResult is the result of allocating devices.
        :param Sequence['DeviceAllocationConfigurationPatchArgs'] config: This field is a combination of all the claim and class configuration parameters. Drivers can distinguish between those based on a flag.
               
               This includes configuration parameters for drivers which have no allocated devices in the result because it is up to the drivers which configuration parameters they support. They can silently ignore unknown configuration parameters.
        :param Sequence['DeviceRequestAllocationResultPatchArgs'] results: Results lists all allocated devices.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if results is not None:
            pulumi.set(__self__, "results", results)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceAllocationConfigurationPatch']]:
        """
        This field is a combination of all the claim and class configuration parameters. Drivers can distinguish between those based on a flag.

        This includes configuration parameters for drivers which have no allocated devices in the result because it is up to the drivers which configuration parameters they support. They can silently ignore unknown configuration parameters.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter
    def results(self) -> Optional[Sequence['outputs.DeviceRequestAllocationResultPatch']]:
        """
        Results lists all allocated devices.
        """
        return pulumi.get(self, "results")


@pulumi.output_type
class DeviceAttribute(dict):
    """
    DeviceAttribute must have exactly one field set.
    """
    def __init__(__self__, *,
                 bool: Optional[_builtins.bool] = None,
                 int: Optional[_builtins.int] = None,
                 string: Optional[_builtins.str] = None,
                 version: Optional[_builtins.str] = None):
        """
        DeviceAttribute must have exactly one field set.
        :param _builtins.bool bool: BoolValue is a true/false value.
        :param _builtins.int int: IntValue is a number.
        :param _builtins.str string: StringValue is a string. Must not be longer than 64 characters.
        :param _builtins.str version: VersionValue is a semantic version according to semver.org spec 2.0.0. Must not be longer than 64 characters.
        """
        if bool is not None:
            pulumi.set(__self__, "bool", bool)
        if int is not None:
            pulumi.set(__self__, "int", int)
        if string is not None:
            pulumi.set(__self__, "string", string)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @_builtins.property
    @pulumi.getter
    def bool(self) -> Optional[_builtins.bool]:
        """
        BoolValue is a true/false value.
        """
        return pulumi.get(self, "bool")

    @_builtins.property
    @pulumi.getter
    def int(self) -> Optional[_builtins.int]:
        """
        IntValue is a number.
        """
        return pulumi.get(self, "int")

    @_builtins.property
    @pulumi.getter
    def string(self) -> Optional[_builtins.str]:
        """
        StringValue is a string. Must not be longer than 64 characters.
        """
        return pulumi.get(self, "string")

    @_builtins.property
    @pulumi.getter
    def version(self) -> Optional[_builtins.str]:
        """
        VersionValue is a semantic version according to semver.org spec 2.0.0. Must not be longer than 64 characters.
        """
        return pulumi.get(self, "version")


@pulumi.output_type
class DeviceCapacity(dict):
    """
    DeviceCapacity describes a quantity associated with a device.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "requestPolicy":
            suggest = "request_policy"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceCapacity. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceCapacity.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceCapacity.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 value: _builtins.str,
                 request_policy: Optional['outputs.CapacityRequestPolicy'] = None):
        """
        DeviceCapacity describes a quantity associated with a device.
        :param _builtins.str value: Value defines how much of a certain capacity that device has.
               
               This field reflects the fixed total capacity and does not change. The consumed amount is tracked separately by scheduler and does not affect this value.
        :param 'CapacityRequestPolicyArgs' request_policy: RequestPolicy defines how this DeviceCapacity must be consumed when the device is allowed to be shared by multiple allocations.
               
               The Device must have allowMultipleAllocations set to true in order to set a requestPolicy.
               
               If unset, capacity requests are unconstrained: requests can consume any amount of capacity, as long as the total consumed across all allocations does not exceed the device's defined capacity. If request is also unset, default is the full capacity value.
        """
        pulumi.set(__self__, "value", value)
        if request_policy is not None:
            pulumi.set(__self__, "request_policy", request_policy)

    @_builtins.property
    @pulumi.getter
    def value(self) -> _builtins.str:
        """
        Value defines how much of a certain capacity that device has.

        This field reflects the fixed total capacity and does not change. The consumed amount is tracked separately by scheduler and does not affect this value.
        """
        return pulumi.get(self, "value")

    @_builtins.property
    @pulumi.getter(name="requestPolicy")
    def request_policy(self) -> Optional['outputs.CapacityRequestPolicy']:
        """
        RequestPolicy defines how this DeviceCapacity must be consumed when the device is allowed to be shared by multiple allocations.

        The Device must have allowMultipleAllocations set to true in order to set a requestPolicy.

        If unset, capacity requests are unconstrained: requests can consume any amount of capacity, as long as the total consumed across all allocations does not exceed the device's defined capacity. If request is also unset, default is the full capacity value.
        """
        return pulumi.get(self, "request_policy")


@pulumi.output_type
class DeviceClaim(dict):
    """
    DeviceClaim defines how to request devices with a ResourceClaim.
    """
    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceClaimConfiguration']] = None,
                 constraints: Optional[Sequence['outputs.DeviceConstraint']] = None,
                 requests: Optional[Sequence['outputs.DeviceRequest']] = None):
        """
        DeviceClaim defines how to request devices with a ResourceClaim.
        :param Sequence['DeviceClaimConfigurationArgs'] config: This field holds configuration for multiple potential drivers which could satisfy requests in this claim. It is ignored while allocating the claim.
        :param Sequence['DeviceConstraintArgs'] constraints: These constraints must be satisfied by the set of devices that get allocated for the claim.
        :param Sequence['DeviceRequestArgs'] requests: Requests represent individual requests for distinct devices which must all be satisfied. If empty, nothing needs to be allocated.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if constraints is not None:
            pulumi.set(__self__, "constraints", constraints)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceClaimConfiguration']]:
        """
        This field holds configuration for multiple potential drivers which could satisfy requests in this claim. It is ignored while allocating the claim.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter
    def constraints(self) -> Optional[Sequence['outputs.DeviceConstraint']]:
        """
        These constraints must be satisfied by the set of devices that get allocated for the claim.
        """
        return pulumi.get(self, "constraints")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence['outputs.DeviceRequest']]:
        """
        Requests represent individual requests for distinct devices which must all be satisfied. If empty, nothing needs to be allocated.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceClaimConfiguration(dict):
    """
    DeviceClaimConfiguration is used for configuration parameters in DeviceClaim.
    """
    def __init__(__self__, *,
                 opaque: Optional['outputs.OpaqueDeviceConfiguration'] = None,
                 requests: Optional[Sequence[_builtins.str]] = None):
        """
        DeviceClaimConfiguration is used for configuration parameters in DeviceClaim.
        :param 'OpaqueDeviceConfigurationArgs' opaque: Opaque provides driver-specific configuration parameters.
        :param Sequence[_builtins.str] requests: Requests lists the names of requests where the configuration applies. If empty, it applies to all requests.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfiguration']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests lists the names of requests where the configuration applies. If empty, it applies to all requests.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceClaimConfigurationPatch(dict):
    """
    DeviceClaimConfiguration is used for configuration parameters in DeviceClaim.
    """
    def __init__(__self__, *,
                 opaque: Optional['outputs.OpaqueDeviceConfigurationPatch'] = None,
                 requests: Optional[Sequence[_builtins.str]] = None):
        """
        DeviceClaimConfiguration is used for configuration parameters in DeviceClaim.
        :param 'OpaqueDeviceConfigurationPatchArgs' opaque: Opaque provides driver-specific configuration parameters.
        :param Sequence[_builtins.str] requests: Requests lists the names of requests where the configuration applies. If empty, it applies to all requests.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfigurationPatch']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests lists the names of requests where the configuration applies. If empty, it applies to all requests.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the configuration applies to all subrequests.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceClaimPatch(dict):
    """
    DeviceClaim defines how to request devices with a ResourceClaim.
    """
    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceClaimConfigurationPatch']] = None,
                 constraints: Optional[Sequence['outputs.DeviceConstraintPatch']] = None,
                 requests: Optional[Sequence['outputs.DeviceRequestPatch']] = None):
        """
        DeviceClaim defines how to request devices with a ResourceClaim.
        :param Sequence['DeviceClaimConfigurationPatchArgs'] config: This field holds configuration for multiple potential drivers which could satisfy requests in this claim. It is ignored while allocating the claim.
        :param Sequence['DeviceConstraintPatchArgs'] constraints: These constraints must be satisfied by the set of devices that get allocated for the claim.
        :param Sequence['DeviceRequestPatchArgs'] requests: Requests represent individual requests for distinct devices which must all be satisfied. If empty, nothing needs to be allocated.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if constraints is not None:
            pulumi.set(__self__, "constraints", constraints)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceClaimConfigurationPatch']]:
        """
        This field holds configuration for multiple potential drivers which could satisfy requests in this claim. It is ignored while allocating the claim.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter
    def constraints(self) -> Optional[Sequence['outputs.DeviceConstraintPatch']]:
        """
        These constraints must be satisfied by the set of devices that get allocated for the claim.
        """
        return pulumi.get(self, "constraints")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence['outputs.DeviceRequestPatch']]:
        """
        Requests represent individual requests for distinct devices which must all be satisfied. If empty, nothing needs to be allocated.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceClass(dict):
    """
    DeviceClass is a vendor- or admin-provided resource that contains device configuration and selectors. It can be referenced in the device requests of a claim to apply these presets. Cluster scoped.

    This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiVersion":
            suggest = "api_version"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceClass. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceClass.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceClass.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 spec: 'outputs.DeviceClassSpec',
                 api_version: Optional[_builtins.str] = None,
                 kind: Optional[_builtins.str] = None,
                 metadata: Optional['_meta.v1.outputs.ObjectMeta'] = None):
        """
        DeviceClass is a vendor- or admin-provided resource that contains device configuration and selectors. It can be referenced in the device requests of a claim to apply these presets. Cluster scoped.

        This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
        :param 'DeviceClassSpecArgs' spec: Spec defines what can be allocated and how to configure it.
               
               This is mutable. Consumers have to be prepared for classes changing at any time, either because they get updated or replaced. Claim allocations are done once based on whatever was set in classes at the time of allocation.
               
               Changing the spec automatically increments the metadata.generation number.
        :param _builtins.str api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param _builtins.str kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param '_meta.v1.ObjectMetaArgs' metadata: Standard object metadata
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'resource.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'DeviceClass')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> 'outputs.DeviceClassSpec':
        """
        Spec defines what can be allocated and how to configure it.

        This is mutable. Consumers have to be prepared for classes changing at any time, either because they get updated or replaced. Claim allocations are done once based on whatever was set in classes at the time of allocation.

        Changing the spec automatically increments the metadata.generation number.
        """
        return pulumi.get(self, "spec")

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[_builtins.str]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[_builtins.str]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMeta']:
        """
        Standard object metadata
        """
        return pulumi.get(self, "metadata")


@pulumi.output_type
class DeviceClassConfiguration(dict):
    """
    DeviceClassConfiguration is used in DeviceClass.
    """
    def __init__(__self__, *,
                 opaque: Optional['outputs.OpaqueDeviceConfiguration'] = None):
        """
        DeviceClassConfiguration is used in DeviceClass.
        :param 'OpaqueDeviceConfigurationArgs' opaque: Opaque provides driver-specific configuration parameters.
        """
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfiguration']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")


@pulumi.output_type
class DeviceClassConfigurationPatch(dict):
    """
    DeviceClassConfiguration is used in DeviceClass.
    """
    def __init__(__self__, *,
                 opaque: Optional['outputs.OpaqueDeviceConfigurationPatch'] = None):
        """
        DeviceClassConfiguration is used in DeviceClass.
        :param 'OpaqueDeviceConfigurationPatchArgs' opaque: Opaque provides driver-specific configuration parameters.
        """
        if opaque is not None:
            pulumi.set(__self__, "opaque", opaque)

    @_builtins.property
    @pulumi.getter
    def opaque(self) -> Optional['outputs.OpaqueDeviceConfigurationPatch']:
        """
        Opaque provides driver-specific configuration parameters.
        """
        return pulumi.get(self, "opaque")


@pulumi.output_type
class DeviceClassSpec(dict):
    """
    DeviceClassSpec is used in a [DeviceClass] to define what can be allocated and how to configure it.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "extendedResourceName":
            suggest = "extended_resource_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceClassSpec. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceClassSpec.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceClassSpec.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceClassConfiguration']] = None,
                 extended_resource_name: Optional[_builtins.str] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelector']] = None):
        """
        DeviceClassSpec is used in a [DeviceClass] to define what can be allocated and how to configure it.
        :param Sequence['DeviceClassConfigurationArgs'] config: Config defines configuration parameters that apply to each device that is claimed via this class. Some classses may potentially be satisfied by multiple drivers, so each instance of a vendor configuration applies to exactly one driver.
               
               They are passed to the driver, but are not considered while allocating the claim.
        :param _builtins.str extended_resource_name: ExtendedResourceName is the extended resource name for the devices of this class. The devices of this class can be used to satisfy a pod's extended resource requests. It has the same format as the name of a pod's extended resource. It should be unique among all the device classes in a cluster. If two device classes have the same name, then the class created later is picked to satisfy a pod's extended resource requests. If two classes are created at the same time, then the name of the class lexicographically sorted first is picked.
               
               This is an alpha field.
        :param Sequence['DeviceSelectorArgs'] selectors: Each selector must be satisfied by a device which is claimed via this class.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if extended_resource_name is not None:
            pulumi.set(__self__, "extended_resource_name", extended_resource_name)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceClassConfiguration']]:
        """
        Config defines configuration parameters that apply to each device that is claimed via this class. Some classses may potentially be satisfied by multiple drivers, so each instance of a vendor configuration applies to exactly one driver.

        They are passed to the driver, but are not considered while allocating the claim.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter(name="extendedResourceName")
    def extended_resource_name(self) -> Optional[_builtins.str]:
        """
        ExtendedResourceName is the extended resource name for the devices of this class. The devices of this class can be used to satisfy a pod's extended resource requests. It has the same format as the name of a pod's extended resource. It should be unique among all the device classes in a cluster. If two device classes have the same name, then the class created later is picked to satisfy a pod's extended resource requests. If two classes are created at the same time, then the name of the class lexicographically sorted first is picked.

        This is an alpha field.
        """
        return pulumi.get(self, "extended_resource_name")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelector']]:
        """
        Each selector must be satisfied by a device which is claimed via this class.
        """
        return pulumi.get(self, "selectors")


@pulumi.output_type
class DeviceClassSpecPatch(dict):
    """
    DeviceClassSpec is used in a [DeviceClass] to define what can be allocated and how to configure it.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "extendedResourceName":
            suggest = "extended_resource_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceClassSpecPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceClassSpecPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceClassSpecPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 config: Optional[Sequence['outputs.DeviceClassConfigurationPatch']] = None,
                 extended_resource_name: Optional[_builtins.str] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelectorPatch']] = None):
        """
        DeviceClassSpec is used in a [DeviceClass] to define what can be allocated and how to configure it.
        :param Sequence['DeviceClassConfigurationPatchArgs'] config: Config defines configuration parameters that apply to each device that is claimed via this class. Some classses may potentially be satisfied by multiple drivers, so each instance of a vendor configuration applies to exactly one driver.
               
               They are passed to the driver, but are not considered while allocating the claim.
        :param _builtins.str extended_resource_name: ExtendedResourceName is the extended resource name for the devices of this class. The devices of this class can be used to satisfy a pod's extended resource requests. It has the same format as the name of a pod's extended resource. It should be unique among all the device classes in a cluster. If two device classes have the same name, then the class created later is picked to satisfy a pod's extended resource requests. If two classes are created at the same time, then the name of the class lexicographically sorted first is picked.
               
               This is an alpha field.
        :param Sequence['DeviceSelectorPatchArgs'] selectors: Each selector must be satisfied by a device which is claimed via this class.
        """
        if config is not None:
            pulumi.set(__self__, "config", config)
        if extended_resource_name is not None:
            pulumi.set(__self__, "extended_resource_name", extended_resource_name)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[Sequence['outputs.DeviceClassConfigurationPatch']]:
        """
        Config defines configuration parameters that apply to each device that is claimed via this class. Some classses may potentially be satisfied by multiple drivers, so each instance of a vendor configuration applies to exactly one driver.

        They are passed to the driver, but are not considered while allocating the claim.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter(name="extendedResourceName")
    def extended_resource_name(self) -> Optional[_builtins.str]:
        """
        ExtendedResourceName is the extended resource name for the devices of this class. The devices of this class can be used to satisfy a pod's extended resource requests. It has the same format as the name of a pod's extended resource. It should be unique among all the device classes in a cluster. If two device classes have the same name, then the class created later is picked to satisfy a pod's extended resource requests. If two classes are created at the same time, then the name of the class lexicographically sorted first is picked.

        This is an alpha field.
        """
        return pulumi.get(self, "extended_resource_name")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelectorPatch']]:
        """
        Each selector must be satisfied by a device which is claimed via this class.
        """
        return pulumi.get(self, "selectors")


@pulumi.output_type
class DeviceConstraint(dict):
    """
    DeviceConstraint must have exactly one field set besides Requests.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "distinctAttribute":
            suggest = "distinct_attribute"
        elif key == "matchAttribute":
            suggest = "match_attribute"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceConstraint. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceConstraint.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceConstraint.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 distinct_attribute: Optional[_builtins.str] = None,
                 match_attribute: Optional[_builtins.str] = None,
                 requests: Optional[Sequence[_builtins.str]] = None):
        """
        DeviceConstraint must have exactly one field set besides Requests.
        :param _builtins.str distinct_attribute: DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.
               
               This acts as the inverse of MatchAttribute.
               
               This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.
               
               This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.
        :param _builtins.str match_attribute: MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.
               
               For example, if you specified "dra.example.com/numa" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.
               
               Must include the domain qualifier.
        :param Sequence[_builtins.str] requests: Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.
        """
        if distinct_attribute is not None:
            pulumi.set(__self__, "distinct_attribute", distinct_attribute)
        if match_attribute is not None:
            pulumi.set(__self__, "match_attribute", match_attribute)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter(name="distinctAttribute")
    def distinct_attribute(self) -> Optional[_builtins.str]:
        """
        DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.

        This acts as the inverse of MatchAttribute.

        This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.

        This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.
        """
        return pulumi.get(self, "distinct_attribute")

    @_builtins.property
    @pulumi.getter(name="matchAttribute")
    def match_attribute(self) -> Optional[_builtins.str]:
        """
        MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.

        For example, if you specified "dra.example.com/numa" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.

        Must include the domain qualifier.
        """
        return pulumi.get(self, "match_attribute")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceConstraintPatch(dict):
    """
    DeviceConstraint must have exactly one field set besides Requests.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "distinctAttribute":
            suggest = "distinct_attribute"
        elif key == "matchAttribute":
            suggest = "match_attribute"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceConstraintPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceConstraintPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceConstraintPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 distinct_attribute: Optional[_builtins.str] = None,
                 match_attribute: Optional[_builtins.str] = None,
                 requests: Optional[Sequence[_builtins.str]] = None):
        """
        DeviceConstraint must have exactly one field set besides Requests.
        :param _builtins.str distinct_attribute: DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.
               
               This acts as the inverse of MatchAttribute.
               
               This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.
               
               This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.
        :param _builtins.str match_attribute: MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.
               
               For example, if you specified "dra.example.com/numa" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.
               
               Must include the domain qualifier.
        :param Sequence[_builtins.str] requests: Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.
               
               References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.
        """
        if distinct_attribute is not None:
            pulumi.set(__self__, "distinct_attribute", distinct_attribute)
        if match_attribute is not None:
            pulumi.set(__self__, "match_attribute", match_attribute)
        if requests is not None:
            pulumi.set(__self__, "requests", requests)

    @_builtins.property
    @pulumi.getter(name="distinctAttribute")
    def distinct_attribute(self) -> Optional[_builtins.str]:
        """
        DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.

        This acts as the inverse of MatchAttribute.

        This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.

        This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.
        """
        return pulumi.get(self, "distinct_attribute")

    @_builtins.property
    @pulumi.getter(name="matchAttribute")
    def match_attribute(self) -> Optional[_builtins.str]:
        """
        MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.

        For example, if you specified "dra.example.com/numa" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.

        Must include the domain qualifier.
        """
        return pulumi.get(self, "match_attribute")

    @_builtins.property
    @pulumi.getter
    def requests(self) -> Optional[Sequence[_builtins.str]]:
        """
        Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.

        References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.
        """
        return pulumi.get(self, "requests")


@pulumi.output_type
class DeviceCounterConsumption(dict):
    """
    DeviceCounterConsumption defines a set of counters that a device will consume from a CounterSet.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "counterSet":
            suggest = "counter_set"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceCounterConsumption. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceCounterConsumption.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceCounterConsumption.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 counter_set: _builtins.str,
                 counters: Mapping[str, 'outputs.Counter']):
        """
        DeviceCounterConsumption defines a set of counters that a device will consume from a CounterSet.
        :param _builtins.str counter_set: CounterSet is the name of the set from which the counters defined will be consumed.
        :param Mapping[str, 'CounterArgs'] counters: Counters defines the counters that will be consumed by the device.
               
               The maximum number of counters is 32.
        """
        pulumi.set(__self__, "counter_set", counter_set)
        pulumi.set(__self__, "counters", counters)

    @_builtins.property
    @pulumi.getter(name="counterSet")
    def counter_set(self) -> _builtins.str:
        """
        CounterSet is the name of the set from which the counters defined will be consumed.
        """
        return pulumi.get(self, "counter_set")

    @_builtins.property
    @pulumi.getter
    def counters(self) -> Mapping[str, 'outputs.Counter']:
        """
        Counters defines the counters that will be consumed by the device.

        The maximum number of counters is 32.
        """
        return pulumi.get(self, "counters")


@pulumi.output_type
class DeviceCounterConsumptionPatch(dict):
    """
    DeviceCounterConsumption defines a set of counters that a device will consume from a CounterSet.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "counterSet":
            suggest = "counter_set"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceCounterConsumptionPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceCounterConsumptionPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceCounterConsumptionPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 counter_set: Optional[_builtins.str] = None,
                 counters: Optional[Mapping[str, 'outputs.Counter']] = None):
        """
        DeviceCounterConsumption defines a set of counters that a device will consume from a CounterSet.
        :param _builtins.str counter_set: CounterSet is the name of the set from which the counters defined will be consumed.
        :param Mapping[str, 'CounterArgs'] counters: Counters defines the counters that will be consumed by the device.
               
               The maximum number of counters is 32.
        """
        if counter_set is not None:
            pulumi.set(__self__, "counter_set", counter_set)
        if counters is not None:
            pulumi.set(__self__, "counters", counters)

    @_builtins.property
    @pulumi.getter(name="counterSet")
    def counter_set(self) -> Optional[_builtins.str]:
        """
        CounterSet is the name of the set from which the counters defined will be consumed.
        """
        return pulumi.get(self, "counter_set")

    @_builtins.property
    @pulumi.getter
    def counters(self) -> Optional[Mapping[str, 'outputs.Counter']]:
        """
        Counters defines the counters that will be consumed by the device.

        The maximum number of counters is 32.
        """
        return pulumi.get(self, "counters")


@pulumi.output_type
class DevicePatch(dict):
    """
    Device represents one individual hardware instance that can be selected based on its attributes. Besides the name, exactly one field must be set.
    """
    def __init__(__self__, *,
                 basic: Optional['outputs.BasicDevicePatch'] = None,
                 name: Optional[_builtins.str] = None):
        """
        Device represents one individual hardware instance that can be selected based on its attributes. Besides the name, exactly one field must be set.
        :param 'BasicDevicePatchArgs' basic: Basic defines one device instance.
        :param _builtins.str name: Name is unique identifier among all devices managed by the driver in the pool. It must be a DNS label.
        """
        if basic is not None:
            pulumi.set(__self__, "basic", basic)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter
    def basic(self) -> Optional['outputs.BasicDevicePatch']:
        """
        Basic defines one device instance.
        """
        return pulumi.get(self, "basic")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name is unique identifier among all devices managed by the driver in the pool. It must be a DNS label.
        """
        return pulumi.get(self, "name")


@pulumi.output_type
class DeviceRequest(dict):
    """
    DeviceRequest is a request for devices required for a claim. This is typically a request for a single resource like a device, but can also ask for several identical devices.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "adminAccess":
            suggest = "admin_access"
        elif key == "allocationMode":
            suggest = "allocation_mode"
        elif key == "deviceClassName":
            suggest = "device_class_name"
        elif key == "firstAvailable":
            suggest = "first_available"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceRequest. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceRequest.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceRequest.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 name: _builtins.str,
                 admin_access: Optional[_builtins.bool] = None,
                 allocation_mode: Optional[_builtins.str] = None,
                 capacity: Optional['outputs.CapacityRequirements'] = None,
                 count: Optional[_builtins.int] = None,
                 device_class_name: Optional[_builtins.str] = None,
                 first_available: Optional[Sequence['outputs.DeviceSubRequest']] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelector']] = None,
                 tolerations: Optional[Sequence['outputs.DeviceToleration']] = None):
        """
        DeviceRequest is a request for devices required for a claim. This is typically a request for a single resource like a device, but can also ask for several identical devices.
        :param _builtins.str name: Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.
               
               Must be a DNS label and unique among all DeviceRequests in a ResourceClaim.
        :param _builtins.bool admin_access: AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        :param _builtins.str allocation_mode: AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:
               
               - ExactCount: This request is for a specific number of devices.
                 This is the default. The exact number is provided in the
                 count field.
               
               - All: This request is for all of the matching devices in a pool.
                 At least one device must exist on the node for the allocation to succeed.
                 Allocation will fail if some devices are already allocated,
                 unless adminAccess is requested.
               
               If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        :param 'CapacityRequirementsArgs' capacity: Capacity define resource requirements against each capacity.
               
               If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.
               
               Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        :param _builtins.int count: Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        :param _builtins.str device_class_name: DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.
               
               A class is required if no subrequests are specified in the firstAvailable list and no class can be set if subrequests are specified in the firstAvailable list. Which classes are available depends on the cluster.
               
               Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        :param Sequence['DeviceSubRequestArgs'] first_available: FirstAvailable contains subrequests, of which exactly one will be satisfied by the scheduler to satisfy this request. It tries to satisfy them in the order in which they are listed here. So if there are two entries in the list, the scheduler will only check the second one if it determines that the first one cannot be used.
               
               This field may only be set in the entries of DeviceClaim.Requests.
               
               DRA does not yet implement scoring, so the scheduler will select the first set of devices that satisfies all the requests in the claim. And if the requirements can be satisfied on more than one node, other scheduling features will determine which node is chosen. This means that the set of devices allocated to a claim might not be the optimal set available to the cluster. Scoring will be implemented later.
        :param Sequence['DeviceSelectorArgs'] selectors: Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        :param Sequence['DeviceTolerationArgs'] tolerations: If specified, the request's tolerations.
               
               Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.
               
               In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.
               
               The maximum number of tolerations is 16.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        pulumi.set(__self__, "name", name)
        if admin_access is not None:
            pulumi.set(__self__, "admin_access", admin_access)
        if allocation_mode is not None:
            pulumi.set(__self__, "allocation_mode", allocation_mode)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if count is not None:
            pulumi.set(__self__, "count", count)
        if device_class_name is not None:
            pulumi.set(__self__, "device_class_name", device_class_name)
        if first_available is not None:
            pulumi.set(__self__, "first_available", first_available)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.

        Must be a DNS label and unique among all DeviceRequests in a ResourceClaim.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="adminAccess")
    def admin_access(self) -> Optional[_builtins.bool]:
        """
        AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        """
        return pulumi.get(self, "admin_access")

    @_builtins.property
    @pulumi.getter(name="allocationMode")
    def allocation_mode(self) -> Optional[_builtins.str]:
        """
        AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:

        - ExactCount: This request is for a specific number of devices.
          This is the default. The exact number is provided in the
          count field.

        - All: This request is for all of the matching devices in a pool.
          At least one device must exist on the node for the allocation to succeed.
          Allocation will fail if some devices are already allocated,
          unless adminAccess is requested.

        If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        """
        return pulumi.get(self, "allocation_mode")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional['outputs.CapacityRequirements']:
        """
        Capacity define resource requirements against each capacity.

        If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.

        Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter
    def count(self) -> Optional[_builtins.int]:
        """
        Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        """
        return pulumi.get(self, "count")

    @_builtins.property
    @pulumi.getter(name="deviceClassName")
    def device_class_name(self) -> Optional[_builtins.str]:
        """
        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.

        A class is required if no subrequests are specified in the firstAvailable list and no class can be set if subrequests are specified in the firstAvailable list. Which classes are available depends on the cluster.

        Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        """
        return pulumi.get(self, "device_class_name")

    @_builtins.property
    @pulumi.getter(name="firstAvailable")
    def first_available(self) -> Optional[Sequence['outputs.DeviceSubRequest']]:
        """
        FirstAvailable contains subrequests, of which exactly one will be satisfied by the scheduler to satisfy this request. It tries to satisfy them in the order in which they are listed here. So if there are two entries in the list, the scheduler will only check the second one if it determines that the first one cannot be used.

        This field may only be set in the entries of DeviceClaim.Requests.

        DRA does not yet implement scoring, so the scheduler will select the first set of devices that satisfies all the requests in the claim. And if the requirements can be satisfied on more than one node, other scheduling features will determine which node is chosen. This means that the set of devices allocated to a claim might not be the optimal set available to the cluster. Scoring will be implemented later.
        """
        return pulumi.get(self, "first_available")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelector']]:
        """
        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        """
        return pulumi.get(self, "selectors")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceToleration']]:
        """
        If specified, the request's tolerations.

        Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.

        In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.

        The maximum number of tolerations is 16.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceRequestAllocationResult(dict):
    """
    DeviceRequestAllocationResult contains the allocation result for one request.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "adminAccess":
            suggest = "admin_access"
        elif key == "bindingConditions":
            suggest = "binding_conditions"
        elif key == "bindingFailureConditions":
            suggest = "binding_failure_conditions"
        elif key == "consumedCapacity":
            suggest = "consumed_capacity"
        elif key == "shareID":
            suggest = "share_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceRequestAllocationResult. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceRequestAllocationResult.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceRequestAllocationResult.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 device: _builtins.str,
                 driver: _builtins.str,
                 pool: _builtins.str,
                 request: _builtins.str,
                 admin_access: Optional[_builtins.bool] = None,
                 binding_conditions: Optional[Sequence[_builtins.str]] = None,
                 binding_failure_conditions: Optional[Sequence[_builtins.str]] = None,
                 consumed_capacity: Optional[Mapping[str, _builtins.str]] = None,
                 share_id: Optional[_builtins.str] = None,
                 tolerations: Optional[Sequence['outputs.DeviceToleration']] = None):
        """
        DeviceRequestAllocationResult contains the allocation result for one request.
        :param _builtins.str device: Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        :param _builtins.str driver: Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param _builtins.str pool: This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).
               
               Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        :param _builtins.str request: Request is the name of the request in the claim which caused this device to be allocated. If it references a subrequest in the firstAvailable list on a DeviceRequest, this field must include both the name of the main request and the subrequest using the format <main request>/<subrequest>.
               
               Multiple devices may have been allocated per request.
        :param _builtins.bool admin_access: AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.
               
               This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        :param Sequence[_builtins.str] binding_conditions: BindingConditions contains a copy of the BindingConditions from the corresponding ResourceSlice at the time of allocation.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Sequence[_builtins.str] binding_failure_conditions: BindingFailureConditions contains a copy of the BindingFailureConditions from the corresponding ResourceSlice at the time of allocation.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Mapping[str, _builtins.str] consumed_capacity: ConsumedCapacity tracks the amount of capacity consumed per device as part of the claim request. The consumed amount may differ from the requested amount: it is rounded up to the nearest valid value based on the device’s requestPolicy if applicable (i.e., may not be less than the requested amount).
               
               The total consumed capacity for each device must not exceed the DeviceCapacity's Value.
               
               This field is populated only for devices that allow multiple allocations. All capacity entries are included, even if the consumed amount is zero.
        :param _builtins.str share_id: ShareID uniquely identifies an individual allocation share of the device, used when the device supports multiple simultaneous allocations. It serves as an additional map key to differentiate concurrent shares of the same device.
        :param Sequence['DeviceTolerationArgs'] tolerations: A copy of all tolerations specified in the request at the time when the device got allocated.
               
               The maximum number of tolerations is 16.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        pulumi.set(__self__, "device", device)
        pulumi.set(__self__, "driver", driver)
        pulumi.set(__self__, "pool", pool)
        pulumi.set(__self__, "request", request)
        if admin_access is not None:
            pulumi.set(__self__, "admin_access", admin_access)
        if binding_conditions is not None:
            pulumi.set(__self__, "binding_conditions", binding_conditions)
        if binding_failure_conditions is not None:
            pulumi.set(__self__, "binding_failure_conditions", binding_failure_conditions)
        if consumed_capacity is not None:
            pulumi.set(__self__, "consumed_capacity", consumed_capacity)
        if share_id is not None:
            pulumi.set(__self__, "share_id", share_id)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter
    def device(self) -> _builtins.str:
        """
        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        """
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter
    def driver(self) -> _builtins.str:
        """
        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> _builtins.str:
        """
        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).

        Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter
    def request(self) -> _builtins.str:
        """
        Request is the name of the request in the claim which caused this device to be allocated. If it references a subrequest in the firstAvailable list on a DeviceRequest, this field must include both the name of the main request and the subrequest using the format <main request>/<subrequest>.

        Multiple devices may have been allocated per request.
        """
        return pulumi.get(self, "request")

    @_builtins.property
    @pulumi.getter(name="adminAccess")
    def admin_access(self) -> Optional[_builtins.bool]:
        """
        AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.

        This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        """
        return pulumi.get(self, "admin_access")

    @_builtins.property
    @pulumi.getter(name="bindingConditions")
    def binding_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingConditions contains a copy of the BindingConditions from the corresponding ResourceSlice at the time of allocation.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_conditions")

    @_builtins.property
    @pulumi.getter(name="bindingFailureConditions")
    def binding_failure_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingFailureConditions contains a copy of the BindingFailureConditions from the corresponding ResourceSlice at the time of allocation.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_failure_conditions")

    @_builtins.property
    @pulumi.getter(name="consumedCapacity")
    def consumed_capacity(self) -> Optional[Mapping[str, _builtins.str]]:
        """
        ConsumedCapacity tracks the amount of capacity consumed per device as part of the claim request. The consumed amount may differ from the requested amount: it is rounded up to the nearest valid value based on the device’s requestPolicy if applicable (i.e., may not be less than the requested amount).

        The total consumed capacity for each device must not exceed the DeviceCapacity's Value.

        This field is populated only for devices that allow multiple allocations. All capacity entries are included, even if the consumed amount is zero.
        """
        return pulumi.get(self, "consumed_capacity")

    @_builtins.property
    @pulumi.getter(name="shareID")
    def share_id(self) -> Optional[_builtins.str]:
        """
        ShareID uniquely identifies an individual allocation share of the device, used when the device supports multiple simultaneous allocations. It serves as an additional map key to differentiate concurrent shares of the same device.
        """
        return pulumi.get(self, "share_id")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceToleration']]:
        """
        A copy of all tolerations specified in the request at the time when the device got allocated.

        The maximum number of tolerations is 16.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceRequestAllocationResultPatch(dict):
    """
    DeviceRequestAllocationResult contains the allocation result for one request.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "adminAccess":
            suggest = "admin_access"
        elif key == "bindingConditions":
            suggest = "binding_conditions"
        elif key == "bindingFailureConditions":
            suggest = "binding_failure_conditions"
        elif key == "consumedCapacity":
            suggest = "consumed_capacity"
        elif key == "shareID":
            suggest = "share_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceRequestAllocationResultPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceRequestAllocationResultPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceRequestAllocationResultPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 admin_access: Optional[_builtins.bool] = None,
                 binding_conditions: Optional[Sequence[_builtins.str]] = None,
                 binding_failure_conditions: Optional[Sequence[_builtins.str]] = None,
                 consumed_capacity: Optional[Mapping[str, _builtins.str]] = None,
                 device: Optional[_builtins.str] = None,
                 driver: Optional[_builtins.str] = None,
                 pool: Optional[_builtins.str] = None,
                 request: Optional[_builtins.str] = None,
                 share_id: Optional[_builtins.str] = None,
                 tolerations: Optional[Sequence['outputs.DeviceTolerationPatch']] = None):
        """
        DeviceRequestAllocationResult contains the allocation result for one request.
        :param _builtins.bool admin_access: AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.
               
               This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        :param Sequence[_builtins.str] binding_conditions: BindingConditions contains a copy of the BindingConditions from the corresponding ResourceSlice at the time of allocation.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Sequence[_builtins.str] binding_failure_conditions: BindingFailureConditions contains a copy of the BindingFailureConditions from the corresponding ResourceSlice at the time of allocation.
               
               This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        :param Mapping[str, _builtins.str] consumed_capacity: ConsumedCapacity tracks the amount of capacity consumed per device as part of the claim request. The consumed amount may differ from the requested amount: it is rounded up to the nearest valid value based on the device’s requestPolicy if applicable (i.e., may not be less than the requested amount).
               
               The total consumed capacity for each device must not exceed the DeviceCapacity's Value.
               
               This field is populated only for devices that allow multiple allocations. All capacity entries are included, even if the consumed amount is zero.
        :param _builtins.str device: Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        :param _builtins.str driver: Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param _builtins.str pool: This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).
               
               Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        :param _builtins.str request: Request is the name of the request in the claim which caused this device to be allocated. If it references a subrequest in the firstAvailable list on a DeviceRequest, this field must include both the name of the main request and the subrequest using the format <main request>/<subrequest>.
               
               Multiple devices may have been allocated per request.
        :param _builtins.str share_id: ShareID uniquely identifies an individual allocation share of the device, used when the device supports multiple simultaneous allocations. It serves as an additional map key to differentiate concurrent shares of the same device.
        :param Sequence['DeviceTolerationPatchArgs'] tolerations: A copy of all tolerations specified in the request at the time when the device got allocated.
               
               The maximum number of tolerations is 16.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        if admin_access is not None:
            pulumi.set(__self__, "admin_access", admin_access)
        if binding_conditions is not None:
            pulumi.set(__self__, "binding_conditions", binding_conditions)
        if binding_failure_conditions is not None:
            pulumi.set(__self__, "binding_failure_conditions", binding_failure_conditions)
        if consumed_capacity is not None:
            pulumi.set(__self__, "consumed_capacity", consumed_capacity)
        if device is not None:
            pulumi.set(__self__, "device", device)
        if driver is not None:
            pulumi.set(__self__, "driver", driver)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if request is not None:
            pulumi.set(__self__, "request", request)
        if share_id is not None:
            pulumi.set(__self__, "share_id", share_id)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter(name="adminAccess")
    def admin_access(self) -> Optional[_builtins.bool]:
        """
        AdminAccess indicates that this device was allocated for administrative access. See the corresponding request field for a definition of mode.

        This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        """
        return pulumi.get(self, "admin_access")

    @_builtins.property
    @pulumi.getter(name="bindingConditions")
    def binding_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingConditions contains a copy of the BindingConditions from the corresponding ResourceSlice at the time of allocation.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_conditions")

    @_builtins.property
    @pulumi.getter(name="bindingFailureConditions")
    def binding_failure_conditions(self) -> Optional[Sequence[_builtins.str]]:
        """
        BindingFailureConditions contains a copy of the BindingFailureConditions from the corresponding ResourceSlice at the time of allocation.

        This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.
        """
        return pulumi.get(self, "binding_failure_conditions")

    @_builtins.property
    @pulumi.getter(name="consumedCapacity")
    def consumed_capacity(self) -> Optional[Mapping[str, _builtins.str]]:
        """
        ConsumedCapacity tracks the amount of capacity consumed per device as part of the claim request. The consumed amount may differ from the requested amount: it is rounded up to the nearest valid value based on the device’s requestPolicy if applicable (i.e., may not be less than the requested amount).

        The total consumed capacity for each device must not exceed the DeviceCapacity's Value.

        This field is populated only for devices that allow multiple allocations. All capacity entries are included, even if the consumed amount is zero.
        """
        return pulumi.get(self, "consumed_capacity")

    @_builtins.property
    @pulumi.getter
    def device(self) -> Optional[_builtins.str]:
        """
        Device references one device instance via its name in the driver's resource pool. It must be a DNS label.
        """
        return pulumi.get(self, "device")

    @_builtins.property
    @pulumi.getter
    def driver(self) -> Optional[_builtins.str]:
        """
        Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> Optional[_builtins.str]:
        """
        This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).

        Must not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter
    def request(self) -> Optional[_builtins.str]:
        """
        Request is the name of the request in the claim which caused this device to be allocated. If it references a subrequest in the firstAvailable list on a DeviceRequest, this field must include both the name of the main request and the subrequest using the format <main request>/<subrequest>.

        Multiple devices may have been allocated per request.
        """
        return pulumi.get(self, "request")

    @_builtins.property
    @pulumi.getter(name="shareID")
    def share_id(self) -> Optional[_builtins.str]:
        """
        ShareID uniquely identifies an individual allocation share of the device, used when the device supports multiple simultaneous allocations. It serves as an additional map key to differentiate concurrent shares of the same device.
        """
        return pulumi.get(self, "share_id")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceTolerationPatch']]:
        """
        A copy of all tolerations specified in the request at the time when the device got allocated.

        The maximum number of tolerations is 16.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceRequestPatch(dict):
    """
    DeviceRequest is a request for devices required for a claim. This is typically a request for a single resource like a device, but can also ask for several identical devices.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "adminAccess":
            suggest = "admin_access"
        elif key == "allocationMode":
            suggest = "allocation_mode"
        elif key == "deviceClassName":
            suggest = "device_class_name"
        elif key == "firstAvailable":
            suggest = "first_available"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceRequestPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceRequestPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceRequestPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 admin_access: Optional[_builtins.bool] = None,
                 allocation_mode: Optional[_builtins.str] = None,
                 capacity: Optional['outputs.CapacityRequirementsPatch'] = None,
                 count: Optional[_builtins.int] = None,
                 device_class_name: Optional[_builtins.str] = None,
                 first_available: Optional[Sequence['outputs.DeviceSubRequestPatch']] = None,
                 name: Optional[_builtins.str] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelectorPatch']] = None,
                 tolerations: Optional[Sequence['outputs.DeviceTolerationPatch']] = None):
        """
        DeviceRequest is a request for devices required for a claim. This is typically a request for a single resource like a device, but can also ask for several identical devices.
        :param _builtins.bool admin_access: AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        :param _builtins.str allocation_mode: AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:
               
               - ExactCount: This request is for a specific number of devices.
                 This is the default. The exact number is provided in the
                 count field.
               
               - All: This request is for all of the matching devices in a pool.
                 At least one device must exist on the node for the allocation to succeed.
                 Allocation will fail if some devices are already allocated,
                 unless adminAccess is requested.
               
               If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        :param 'CapacityRequirementsPatchArgs' capacity: Capacity define resource requirements against each capacity.
               
               If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.
               
               Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        :param _builtins.int count: Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        :param _builtins.str device_class_name: DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.
               
               A class is required if no subrequests are specified in the firstAvailable list and no class can be set if subrequests are specified in the firstAvailable list. Which classes are available depends on the cluster.
               
               Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        :param Sequence['DeviceSubRequestPatchArgs'] first_available: FirstAvailable contains subrequests, of which exactly one will be satisfied by the scheduler to satisfy this request. It tries to satisfy them in the order in which they are listed here. So if there are two entries in the list, the scheduler will only check the second one if it determines that the first one cannot be used.
               
               This field may only be set in the entries of DeviceClaim.Requests.
               
               DRA does not yet implement scoring, so the scheduler will select the first set of devices that satisfies all the requests in the claim. And if the requirements can be satisfied on more than one node, other scheduling features will determine which node is chosen. This means that the set of devices allocated to a claim might not be the optimal set available to the cluster. Scoring will be implemented later.
        :param _builtins.str name: Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.
               
               Must be a DNS label and unique among all DeviceRequests in a ResourceClaim.
        :param Sequence['DeviceSelectorPatchArgs'] selectors: Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        :param Sequence['DeviceTolerationPatchArgs'] tolerations: If specified, the request's tolerations.
               
               Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.
               
               In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.
               
               The maximum number of tolerations is 16.
               
               This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        if admin_access is not None:
            pulumi.set(__self__, "admin_access", admin_access)
        if allocation_mode is not None:
            pulumi.set(__self__, "allocation_mode", allocation_mode)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if count is not None:
            pulumi.set(__self__, "count", count)
        if device_class_name is not None:
            pulumi.set(__self__, "device_class_name", device_class_name)
        if first_available is not None:
            pulumi.set(__self__, "first_available", first_available)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter(name="adminAccess")
    def admin_access(self) -> Optional[_builtins.bool]:
        """
        AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        This is an alpha field and requires enabling the DRAAdminAccess feature gate. Admin access is disabled if this field is unset or set to false, otherwise it is enabled.
        """
        return pulumi.get(self, "admin_access")

    @_builtins.property
    @pulumi.getter(name="allocationMode")
    def allocation_mode(self) -> Optional[_builtins.str]:
        """
        AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:

        - ExactCount: This request is for a specific number of devices.
          This is the default. The exact number is provided in the
          count field.

        - All: This request is for all of the matching devices in a pool.
          At least one device must exist on the node for the allocation to succeed.
          Allocation will fail if some devices are already allocated,
          unless adminAccess is requested.

        If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        """
        return pulumi.get(self, "allocation_mode")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional['outputs.CapacityRequirementsPatch']:
        """
        Capacity define resource requirements against each capacity.

        If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.

        Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter
    def count(self) -> Optional[_builtins.int]:
        """
        Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        """
        return pulumi.get(self, "count")

    @_builtins.property
    @pulumi.getter(name="deviceClassName")
    def device_class_name(self) -> Optional[_builtins.str]:
        """
        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.

        A class is required if no subrequests are specified in the firstAvailable list and no class can be set if subrequests are specified in the firstAvailable list. Which classes are available depends on the cluster.

        Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        """
        return pulumi.get(self, "device_class_name")

    @_builtins.property
    @pulumi.getter(name="firstAvailable")
    def first_available(self) -> Optional[Sequence['outputs.DeviceSubRequestPatch']]:
        """
        FirstAvailable contains subrequests, of which exactly one will be satisfied by the scheduler to satisfy this request. It tries to satisfy them in the order in which they are listed here. So if there are two entries in the list, the scheduler will only check the second one if it determines that the first one cannot be used.

        This field may only be set in the entries of DeviceClaim.Requests.

        DRA does not yet implement scoring, so the scheduler will select the first set of devices that satisfies all the requests in the claim. And if the requirements can be satisfied on more than one node, other scheduling features will determine which node is chosen. This means that the set of devices allocated to a claim might not be the optimal set available to the cluster. Scoring will be implemented later.
        """
        return pulumi.get(self, "first_available")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.

        Must be a DNS label and unique among all DeviceRequests in a ResourceClaim.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelectorPatch']]:
        """
        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.
        """
        return pulumi.get(self, "selectors")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceTolerationPatch']]:
        """
        If specified, the request's tolerations.

        Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.

        In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.

        The maximum number of tolerations is 16.

        This field can only be set when deviceClassName is set and no subrequests are specified in the firstAvailable list.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceSelector(dict):
    """
    DeviceSelector must have exactly one field set.
    """
    def __init__(__self__, *,
                 cel: Optional['outputs.CELDeviceSelector'] = None):
        """
        DeviceSelector must have exactly one field set.
        :param 'CELDeviceSelectorArgs' cel: CEL contains a CEL expression for selecting a device.
        """
        if cel is not None:
            pulumi.set(__self__, "cel", cel)

    @_builtins.property
    @pulumi.getter
    def cel(self) -> Optional['outputs.CELDeviceSelector']:
        """
        CEL contains a CEL expression for selecting a device.
        """
        return pulumi.get(self, "cel")


@pulumi.output_type
class DeviceSelectorPatch(dict):
    """
    DeviceSelector must have exactly one field set.
    """
    def __init__(__self__, *,
                 cel: Optional['outputs.CELDeviceSelectorPatch'] = None):
        """
        DeviceSelector must have exactly one field set.
        :param 'CELDeviceSelectorPatchArgs' cel: CEL contains a CEL expression for selecting a device.
        """
        if cel is not None:
            pulumi.set(__self__, "cel", cel)

    @_builtins.property
    @pulumi.getter
    def cel(self) -> Optional['outputs.CELDeviceSelectorPatch']:
        """
        CEL contains a CEL expression for selecting a device.
        """
        return pulumi.get(self, "cel")


@pulumi.output_type
class DeviceSubRequest(dict):
    """
    DeviceSubRequest describes a request for device provided in the claim.spec.devices.requests[].firstAvailable array. Each is typically a request for a single resource like a device, but can also ask for several identical devices.

    DeviceSubRequest is similar to Request, but doesn't expose the AdminAccess or FirstAvailable fields, as those can only be set on the top-level request. AdminAccess is not supported for requests with a prioritized list, and recursive FirstAvailable fields are not supported.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "deviceClassName":
            suggest = "device_class_name"
        elif key == "allocationMode":
            suggest = "allocation_mode"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceSubRequest. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceSubRequest.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceSubRequest.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 device_class_name: _builtins.str,
                 name: _builtins.str,
                 allocation_mode: Optional[_builtins.str] = None,
                 capacity: Optional['outputs.CapacityRequirements'] = None,
                 count: Optional[_builtins.int] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelector']] = None,
                 tolerations: Optional[Sequence['outputs.DeviceToleration']] = None):
        """
        DeviceSubRequest describes a request for device provided in the claim.spec.devices.requests[].firstAvailable array. Each is typically a request for a single resource like a device, but can also ask for several identical devices.

        DeviceSubRequest is similar to Request, but doesn't expose the AdminAccess or FirstAvailable fields, as those can only be set on the top-level request. AdminAccess is not supported for requests with a prioritized list, and recursive FirstAvailable fields are not supported.
        :param _builtins.str device_class_name: DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this subrequest.
               
               A class is required. Which classes are available depends on the cluster.
               
               Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        :param _builtins.str name: Name can be used to reference this subrequest in the list of constraints or the list of configurations for the claim. References must use the format <main request>/<subrequest>.
               
               Must be a DNS label.
        :param _builtins.str allocation_mode: AllocationMode and its related fields define how devices are allocated to satisfy this subrequest. Supported values are:
               
               - ExactCount: This request is for a specific number of devices.
                 This is the default. The exact number is provided in the
                 count field.
               
               - All: This subrequest is for all of the matching devices in a pool.
                 Allocation will fail if some devices are already allocated,
                 unless adminAccess is requested.
               
               If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other subrequests must specify this field.
               
               More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        :param 'CapacityRequirementsArgs' capacity: Capacity define resource requirements against each capacity.
               
               If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.
               
               Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        :param _builtins.int count: Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
        :param Sequence['DeviceSelectorArgs'] selectors: Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this subrequest. All selectors must be satisfied for a device to be considered.
        :param Sequence['DeviceTolerationArgs'] tolerations: If specified, the request's tolerations.
               
               Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.
               
               In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.
               
               The maximum number of tolerations is 16.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        pulumi.set(__self__, "device_class_name", device_class_name)
        pulumi.set(__self__, "name", name)
        if allocation_mode is not None:
            pulumi.set(__self__, "allocation_mode", allocation_mode)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if count is not None:
            pulumi.set(__self__, "count", count)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter(name="deviceClassName")
    def device_class_name(self) -> _builtins.str:
        """
        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this subrequest.

        A class is required. Which classes are available depends on the cluster.

        Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        """
        return pulumi.get(self, "device_class_name")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name can be used to reference this subrequest in the list of constraints or the list of configurations for the claim. References must use the format <main request>/<subrequest>.

        Must be a DNS label.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="allocationMode")
    def allocation_mode(self) -> Optional[_builtins.str]:
        """
        AllocationMode and its related fields define how devices are allocated to satisfy this subrequest. Supported values are:

        - ExactCount: This request is for a specific number of devices.
          This is the default. The exact number is provided in the
          count field.

        - All: This subrequest is for all of the matching devices in a pool.
          Allocation will fail if some devices are already allocated,
          unless adminAccess is requested.

        If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other subrequests must specify this field.

        More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        """
        return pulumi.get(self, "allocation_mode")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional['outputs.CapacityRequirements']:
        """
        Capacity define resource requirements against each capacity.

        If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.

        Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter
    def count(self) -> Optional[_builtins.int]:
        """
        Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
        """
        return pulumi.get(self, "count")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelector']]:
        """
        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this subrequest. All selectors must be satisfied for a device to be considered.
        """
        return pulumi.get(self, "selectors")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceToleration']]:
        """
        If specified, the request's tolerations.

        Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.

        In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.

        The maximum number of tolerations is 16.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceSubRequestPatch(dict):
    """
    DeviceSubRequest describes a request for device provided in the claim.spec.devices.requests[].firstAvailable array. Each is typically a request for a single resource like a device, but can also ask for several identical devices.

    DeviceSubRequest is similar to Request, but doesn't expose the AdminAccess or FirstAvailable fields, as those can only be set on the top-level request. AdminAccess is not supported for requests with a prioritized list, and recursive FirstAvailable fields are not supported.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allocationMode":
            suggest = "allocation_mode"
        elif key == "deviceClassName":
            suggest = "device_class_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceSubRequestPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceSubRequestPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceSubRequestPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 allocation_mode: Optional[_builtins.str] = None,
                 capacity: Optional['outputs.CapacityRequirementsPatch'] = None,
                 count: Optional[_builtins.int] = None,
                 device_class_name: Optional[_builtins.str] = None,
                 name: Optional[_builtins.str] = None,
                 selectors: Optional[Sequence['outputs.DeviceSelectorPatch']] = None,
                 tolerations: Optional[Sequence['outputs.DeviceTolerationPatch']] = None):
        """
        DeviceSubRequest describes a request for device provided in the claim.spec.devices.requests[].firstAvailable array. Each is typically a request for a single resource like a device, but can also ask for several identical devices.

        DeviceSubRequest is similar to Request, but doesn't expose the AdminAccess or FirstAvailable fields, as those can only be set on the top-level request. AdminAccess is not supported for requests with a prioritized list, and recursive FirstAvailable fields are not supported.
        :param _builtins.str allocation_mode: AllocationMode and its related fields define how devices are allocated to satisfy this subrequest. Supported values are:
               
               - ExactCount: This request is for a specific number of devices.
                 This is the default. The exact number is provided in the
                 count field.
               
               - All: This subrequest is for all of the matching devices in a pool.
                 Allocation will fail if some devices are already allocated,
                 unless adminAccess is requested.
               
               If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other subrequests must specify this field.
               
               More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        :param 'CapacityRequirementsPatchArgs' capacity: Capacity define resource requirements against each capacity.
               
               If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.
               
               Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        :param _builtins.int count: Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
        :param _builtins.str device_class_name: DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this subrequest.
               
               A class is required. Which classes are available depends on the cluster.
               
               Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        :param _builtins.str name: Name can be used to reference this subrequest in the list of constraints or the list of configurations for the claim. References must use the format <main request>/<subrequest>.
               
               Must be a DNS label.
        :param Sequence['DeviceSelectorPatchArgs'] selectors: Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this subrequest. All selectors must be satisfied for a device to be considered.
        :param Sequence['DeviceTolerationPatchArgs'] tolerations: If specified, the request's tolerations.
               
               Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.
               
               In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.
               
               The maximum number of tolerations is 16.
               
               This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        if allocation_mode is not None:
            pulumi.set(__self__, "allocation_mode", allocation_mode)
        if capacity is not None:
            pulumi.set(__self__, "capacity", capacity)
        if count is not None:
            pulumi.set(__self__, "count", count)
        if device_class_name is not None:
            pulumi.set(__self__, "device_class_name", device_class_name)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if selectors is not None:
            pulumi.set(__self__, "selectors", selectors)
        if tolerations is not None:
            pulumi.set(__self__, "tolerations", tolerations)

    @_builtins.property
    @pulumi.getter(name="allocationMode")
    def allocation_mode(self) -> Optional[_builtins.str]:
        """
        AllocationMode and its related fields define how devices are allocated to satisfy this subrequest. Supported values are:

        - ExactCount: This request is for a specific number of devices.
          This is the default. The exact number is provided in the
          count field.

        - All: This subrequest is for all of the matching devices in a pool.
          Allocation will fail if some devices are already allocated,
          unless adminAccess is requested.

        If AllocationMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other subrequests must specify this field.

        More modes may get added in the future. Clients must refuse to handle requests with unknown modes.
        """
        return pulumi.get(self, "allocation_mode")

    @_builtins.property
    @pulumi.getter
    def capacity(self) -> Optional['outputs.CapacityRequirementsPatch']:
        """
        Capacity define resource requirements against each capacity.

        If this field is unset and the device supports multiple allocations, the default value will be applied to each capacity according to requestPolicy. For the capacity that has no requestPolicy, default is the full capacity value.

        Applies to each device allocation. If Count > 1, the request fails if there aren't enough devices that meet the requirements. If AllocationMode is set to All, the request fails if there are devices that otherwise match the request, and have this capacity, with a value >= the requested amount, but which cannot be allocated to this request.
        """
        return pulumi.get(self, "capacity")

    @_builtins.property
    @pulumi.getter
    def count(self) -> Optional[_builtins.int]:
        """
        Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.
        """
        return pulumi.get(self, "count")

    @_builtins.property
    @pulumi.getter(name="deviceClassName")
    def device_class_name(self) -> Optional[_builtins.str]:
        """
        DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this subrequest.

        A class is required. Which classes are available depends on the cluster.

        Administrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.
        """
        return pulumi.get(self, "device_class_name")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name can be used to reference this subrequest in the list of constraints or the list of configurations for the claim. References must use the format <main request>/<subrequest>.

        Must be a DNS label.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def selectors(self) -> Optional[Sequence['outputs.DeviceSelectorPatch']]:
        """
        Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this subrequest. All selectors must be satisfied for a device to be considered.
        """
        return pulumi.get(self, "selectors")

    @_builtins.property
    @pulumi.getter
    def tolerations(self) -> Optional[Sequence['outputs.DeviceTolerationPatch']]:
        """
        If specified, the request's tolerations.

        Tolerations for NoSchedule are required to allocate a device which has a taint with that effect. The same applies to NoExecute.

        In addition, should any of the allocated devices get tainted with NoExecute after allocation and that effect is not tolerated, then all pods consuming the ResourceClaim get deleted to evict them. The scheduler will not let new pods reserve the claim while it has these tainted devices. Once all pods are evicted, the claim will get deallocated.

        The maximum number of tolerations is 16.

        This is an alpha field and requires enabling the DRADeviceTaints feature gate.
        """
        return pulumi.get(self, "tolerations")


@pulumi.output_type
class DeviceTaint(dict):
    """
    The device this taint is attached to has the "effect" on any claim which does not tolerate the taint and, through the claim, to pods using the claim.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "timeAdded":
            suggest = "time_added"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceTaint. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceTaint.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceTaint.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 effect: _builtins.str,
                 key: _builtins.str,
                 time_added: Optional[_builtins.str] = None,
                 value: Optional[_builtins.str] = None):
        """
        The device this taint is attached to has the "effect" on any claim which does not tolerate the taint and, through the claim, to pods using the claim.
        :param _builtins.str effect: The effect of the taint on claims that do not tolerate the taint and through such claims on the pods using them.
               
               Valid effects are None, NoSchedule and NoExecute. PreferNoSchedule as used for nodes is not valid here. More effects may get added in the future. Consumers must treat unknown effects like None.
        :param _builtins.str key: The taint key to be applied to a device. Must be a label name.
        :param _builtins.str time_added: TimeAdded represents the time at which the taint was added. Added automatically during create or update if not set.
        :param _builtins.str value: The taint value corresponding to the taint key. Must be a label value.
        """
        pulumi.set(__self__, "effect", effect)
        pulumi.set(__self__, "key", key)
        if time_added is not None:
            pulumi.set(__self__, "time_added", time_added)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def effect(self) -> _builtins.str:
        """
        The effect of the taint on claims that do not tolerate the taint and through such claims on the pods using them.

        Valid effects are None, NoSchedule and NoExecute. PreferNoSchedule as used for nodes is not valid here. More effects may get added in the future. Consumers must treat unknown effects like None.
        """
        return pulumi.get(self, "effect")

    @_builtins.property
    @pulumi.getter
    def key(self) -> _builtins.str:
        """
        The taint key to be applied to a device. Must be a label name.
        """
        return pulumi.get(self, "key")

    @_builtins.property
    @pulumi.getter(name="timeAdded")
    def time_added(self) -> Optional[_builtins.str]:
        """
        TimeAdded represents the time at which the taint was added. Added automatically during create or update if not set.
        """
        return pulumi.get(self, "time_added")

    @_builtins.property
    @pulumi.getter
    def value(self) -> Optional[_builtins.str]:
        """
        The taint value corresponding to the taint key. Must be a label value.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class DeviceTaintPatch(dict):
    """
    The device this taint is attached to has the "effect" on any claim which does not tolerate the taint and, through the claim, to pods using the claim.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "timeAdded":
            suggest = "time_added"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceTaintPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceTaintPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceTaintPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 effect: Optional[_builtins.str] = None,
                 key: Optional[_builtins.str] = None,
                 time_added: Optional[_builtins.str] = None,
                 value: Optional[_builtins.str] = None):
        """
        The device this taint is attached to has the "effect" on any claim which does not tolerate the taint and, through the claim, to pods using the claim.
        :param _builtins.str effect: The effect of the taint on claims that do not tolerate the taint and through such claims on the pods using them.
               
               Valid effects are None, NoSchedule and NoExecute. PreferNoSchedule as used for nodes is not valid here. More effects may get added in the future. Consumers must treat unknown effects like None.
        :param _builtins.str key: The taint key to be applied to a device. Must be a label name.
        :param _builtins.str time_added: TimeAdded represents the time at which the taint was added. Added automatically during create or update if not set.
        :param _builtins.str value: The taint value corresponding to the taint key. Must be a label value.
        """
        if effect is not None:
            pulumi.set(__self__, "effect", effect)
        if key is not None:
            pulumi.set(__self__, "key", key)
        if time_added is not None:
            pulumi.set(__self__, "time_added", time_added)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def effect(self) -> Optional[_builtins.str]:
        """
        The effect of the taint on claims that do not tolerate the taint and through such claims on the pods using them.

        Valid effects are None, NoSchedule and NoExecute. PreferNoSchedule as used for nodes is not valid here. More effects may get added in the future. Consumers must treat unknown effects like None.
        """
        return pulumi.get(self, "effect")

    @_builtins.property
    @pulumi.getter
    def key(self) -> Optional[_builtins.str]:
        """
        The taint key to be applied to a device. Must be a label name.
        """
        return pulumi.get(self, "key")

    @_builtins.property
    @pulumi.getter(name="timeAdded")
    def time_added(self) -> Optional[_builtins.str]:
        """
        TimeAdded represents the time at which the taint was added. Added automatically during create or update if not set.
        """
        return pulumi.get(self, "time_added")

    @_builtins.property
    @pulumi.getter
    def value(self) -> Optional[_builtins.str]:
        """
        The taint value corresponding to the taint key. Must be a label value.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class DeviceToleration(dict):
    """
    The ResourceClaim this DeviceToleration is attached to tolerates any taint that matches the triple <key,value,effect> using the matching operator <operator>.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "tolerationSeconds":
            suggest = "toleration_seconds"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceToleration. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceToleration.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceToleration.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 effect: Optional[_builtins.str] = None,
                 key: Optional[_builtins.str] = None,
                 operator: Optional[_builtins.str] = None,
                 toleration_seconds: Optional[_builtins.int] = None,
                 value: Optional[_builtins.str] = None):
        """
        The ResourceClaim this DeviceToleration is attached to tolerates any taint that matches the triple <key,value,effect> using the matching operator <operator>.
        :param _builtins.str effect: Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.
        :param _builtins.str key: Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.
        :param _builtins.str operator: Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.
        :param _builtins.int toleration_seconds: TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.
        :param _builtins.str value: Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.
        """
        if effect is not None:
            pulumi.set(__self__, "effect", effect)
        if key is not None:
            pulumi.set(__self__, "key", key)
        if operator is not None:
            pulumi.set(__self__, "operator", operator)
        if toleration_seconds is not None:
            pulumi.set(__self__, "toleration_seconds", toleration_seconds)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def effect(self) -> Optional[_builtins.str]:
        """
        Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.
        """
        return pulumi.get(self, "effect")

    @_builtins.property
    @pulumi.getter
    def key(self) -> Optional[_builtins.str]:
        """
        Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.
        """
        return pulumi.get(self, "key")

    @_builtins.property
    @pulumi.getter
    def operator(self) -> Optional[_builtins.str]:
        """
        Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.
        """
        return pulumi.get(self, "operator")

    @_builtins.property
    @pulumi.getter(name="tolerationSeconds")
    def toleration_seconds(self) -> Optional[_builtins.int]:
        """
        TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.
        """
        return pulumi.get(self, "toleration_seconds")

    @_builtins.property
    @pulumi.getter
    def value(self) -> Optional[_builtins.str]:
        """
        Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class DeviceTolerationPatch(dict):
    """
    The ResourceClaim this DeviceToleration is attached to tolerates any taint that matches the triple <key,value,effect> using the matching operator <operator>.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "tolerationSeconds":
            suggest = "toleration_seconds"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in DeviceTolerationPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        DeviceTolerationPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        DeviceTolerationPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 effect: Optional[_builtins.str] = None,
                 key: Optional[_builtins.str] = None,
                 operator: Optional[_builtins.str] = None,
                 toleration_seconds: Optional[_builtins.int] = None,
                 value: Optional[_builtins.str] = None):
        """
        The ResourceClaim this DeviceToleration is attached to tolerates any taint that matches the triple <key,value,effect> using the matching operator <operator>.
        :param _builtins.str effect: Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.
        :param _builtins.str key: Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.
        :param _builtins.str operator: Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.
        :param _builtins.int toleration_seconds: TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.
        :param _builtins.str value: Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.
        """
        if effect is not None:
            pulumi.set(__self__, "effect", effect)
        if key is not None:
            pulumi.set(__self__, "key", key)
        if operator is not None:
            pulumi.set(__self__, "operator", operator)
        if toleration_seconds is not None:
            pulumi.set(__self__, "toleration_seconds", toleration_seconds)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def effect(self) -> Optional[_builtins.str]:
        """
        Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.
        """
        return pulumi.get(self, "effect")

    @_builtins.property
    @pulumi.getter
    def key(self) -> Optional[_builtins.str]:
        """
        Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.
        """
        return pulumi.get(self, "key")

    @_builtins.property
    @pulumi.getter
    def operator(self) -> Optional[_builtins.str]:
        """
        Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.
        """
        return pulumi.get(self, "operator")

    @_builtins.property
    @pulumi.getter(name="tolerationSeconds")
    def toleration_seconds(self) -> Optional[_builtins.int]:
        """
        TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.
        """
        return pulumi.get(self, "toleration_seconds")

    @_builtins.property
    @pulumi.getter
    def value(self) -> Optional[_builtins.str]:
        """
        Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.
        """
        return pulumi.get(self, "value")


@pulumi.output_type
class NetworkDeviceData(dict):
    """
    NetworkDeviceData provides network-related details for the allocated device. This information may be filled by drivers or other components to configure or identify the device within a network context.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "hardwareAddress":
            suggest = "hardware_address"
        elif key == "interfaceName":
            suggest = "interface_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in NetworkDeviceData. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        NetworkDeviceData.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        NetworkDeviceData.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 hardware_address: Optional[_builtins.str] = None,
                 interface_name: Optional[_builtins.str] = None,
                 ips: Optional[Sequence[_builtins.str]] = None):
        """
        NetworkDeviceData provides network-related details for the allocated device. This information may be filled by drivers or other components to configure or identify the device within a network context.
        :param _builtins.str hardware_address: HardwareAddress represents the hardware address (e.g. MAC Address) of the device's network interface.
               
               Must not be longer than 128 characters.
        :param _builtins.str interface_name: InterfaceName specifies the name of the network interface associated with the allocated device. This might be the name of a physical or virtual network interface being configured in the pod.
               
               Must not be longer than 256 characters.
        :param Sequence[_builtins.str] ips: IPs lists the network addresses assigned to the device's network interface. This can include both IPv4 and IPv6 addresses. The IPs are in the CIDR notation, which includes both the address and the associated subnet mask. e.g.: "192.0.2.5/24" for IPv4 and "2001:db8::5/64" for IPv6.
               
               Must not contain more than 16 entries.
        """
        if hardware_address is not None:
            pulumi.set(__self__, "hardware_address", hardware_address)
        if interface_name is not None:
            pulumi.set(__self__, "interface_name", interface_name)
        if ips is not None:
            pulumi.set(__self__, "ips", ips)

    @_builtins.property
    @pulumi.getter(name="hardwareAddress")
    def hardware_address(self) -> Optional[_builtins.str]:
        """
        HardwareAddress represents the hardware address (e.g. MAC Address) of the device's network interface.

        Must not be longer than 128 characters.
        """
        return pulumi.get(self, "hardware_address")

    @_builtins.property
    @pulumi.getter(name="interfaceName")
    def interface_name(self) -> Optional[_builtins.str]:
        """
        InterfaceName specifies the name of the network interface associated with the allocated device. This might be the name of a physical or virtual network interface being configured in the pod.

        Must not be longer than 256 characters.
        """
        return pulumi.get(self, "interface_name")

    @_builtins.property
    @pulumi.getter
    def ips(self) -> Optional[Sequence[_builtins.str]]:
        """
        IPs lists the network addresses assigned to the device's network interface. This can include both IPv4 and IPv6 addresses. The IPs are in the CIDR notation, which includes both the address and the associated subnet mask. e.g.: "192.0.2.5/24" for IPv4 and "2001:db8::5/64" for IPv6.

        Must not contain more than 16 entries.
        """
        return pulumi.get(self, "ips")


@pulumi.output_type
class NetworkDeviceDataPatch(dict):
    """
    NetworkDeviceData provides network-related details for the allocated device. This information may be filled by drivers or other components to configure or identify the device within a network context.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "hardwareAddress":
            suggest = "hardware_address"
        elif key == "interfaceName":
            suggest = "interface_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in NetworkDeviceDataPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        NetworkDeviceDataPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        NetworkDeviceDataPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 hardware_address: Optional[_builtins.str] = None,
                 interface_name: Optional[_builtins.str] = None,
                 ips: Optional[Sequence[_builtins.str]] = None):
        """
        NetworkDeviceData provides network-related details for the allocated device. This information may be filled by drivers or other components to configure or identify the device within a network context.
        :param _builtins.str hardware_address: HardwareAddress represents the hardware address (e.g. MAC Address) of the device's network interface.
               
               Must not be longer than 128 characters.
        :param _builtins.str interface_name: InterfaceName specifies the name of the network interface associated with the allocated device. This might be the name of a physical or virtual network interface being configured in the pod.
               
               Must not be longer than 256 characters.
        :param Sequence[_builtins.str] ips: IPs lists the network addresses assigned to the device's network interface. This can include both IPv4 and IPv6 addresses. The IPs are in the CIDR notation, which includes both the address and the associated subnet mask. e.g.: "192.0.2.5/24" for IPv4 and "2001:db8::5/64" for IPv6.
               
               Must not contain more than 16 entries.
        """
        if hardware_address is not None:
            pulumi.set(__self__, "hardware_address", hardware_address)
        if interface_name is not None:
            pulumi.set(__self__, "interface_name", interface_name)
        if ips is not None:
            pulumi.set(__self__, "ips", ips)

    @_builtins.property
    @pulumi.getter(name="hardwareAddress")
    def hardware_address(self) -> Optional[_builtins.str]:
        """
        HardwareAddress represents the hardware address (e.g. MAC Address) of the device's network interface.

        Must not be longer than 128 characters.
        """
        return pulumi.get(self, "hardware_address")

    @_builtins.property
    @pulumi.getter(name="interfaceName")
    def interface_name(self) -> Optional[_builtins.str]:
        """
        InterfaceName specifies the name of the network interface associated with the allocated device. This might be the name of a physical or virtual network interface being configured in the pod.

        Must not be longer than 256 characters.
        """
        return pulumi.get(self, "interface_name")

    @_builtins.property
    @pulumi.getter
    def ips(self) -> Optional[Sequence[_builtins.str]]:
        """
        IPs lists the network addresses assigned to the device's network interface. This can include both IPv4 and IPv6 addresses. The IPs are in the CIDR notation, which includes both the address and the associated subnet mask. e.g.: "192.0.2.5/24" for IPv4 and "2001:db8::5/64" for IPv6.

        Must not contain more than 16 entries.
        """
        return pulumi.get(self, "ips")


@pulumi.output_type
class OpaqueDeviceConfiguration(dict):
    """
    OpaqueDeviceConfiguration contains configuration parameters for a driver in a format defined by the driver vendor.
    """
    def __init__(__self__, *,
                 driver: _builtins.str,
                 parameters: Any):
        """
        OpaqueDeviceConfiguration contains configuration parameters for a driver in a format defined by the driver vendor.
        :param _builtins.str driver: Driver is used to determine which kubelet plugin needs to be passed these configuration parameters.
               
               An admission policy provided by the driver developer could use this to decide whether it needs to validate them.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param Any parameters: Parameters can contain arbitrary data. It is the responsibility of the driver developer to handle validation and versioning. Typically this includes self-identification and a version ("kind" + "apiVersion" for Kubernetes types), with conversion between different versions.
               
               The length of the raw data must be smaller or equal to 10 Ki.
        """
        pulumi.set(__self__, "driver", driver)
        pulumi.set(__self__, "parameters", parameters)

    @_builtins.property
    @pulumi.getter
    def driver(self) -> _builtins.str:
        """
        Driver is used to determine which kubelet plugin needs to be passed these configuration parameters.

        An admission policy provided by the driver developer could use this to decide whether it needs to validate them.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def parameters(self) -> Any:
        """
        Parameters can contain arbitrary data. It is the responsibility of the driver developer to handle validation and versioning. Typically this includes self-identification and a version ("kind" + "apiVersion" for Kubernetes types), with conversion between different versions.

        The length of the raw data must be smaller or equal to 10 Ki.
        """
        return pulumi.get(self, "parameters")


@pulumi.output_type
class OpaqueDeviceConfigurationPatch(dict):
    """
    OpaqueDeviceConfiguration contains configuration parameters for a driver in a format defined by the driver vendor.
    """
    def __init__(__self__, *,
                 driver: Optional[_builtins.str] = None,
                 parameters: Optional[Any] = None):
        """
        OpaqueDeviceConfiguration contains configuration parameters for a driver in a format defined by the driver vendor.
        :param _builtins.str driver: Driver is used to determine which kubelet plugin needs to be passed these configuration parameters.
               
               An admission policy provided by the driver developer could use this to decide whether it needs to validate them.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        :param Any parameters: Parameters can contain arbitrary data. It is the responsibility of the driver developer to handle validation and versioning. Typically this includes self-identification and a version ("kind" + "apiVersion" for Kubernetes types), with conversion between different versions.
               
               The length of the raw data must be smaller or equal to 10 Ki.
        """
        if driver is not None:
            pulumi.set(__self__, "driver", driver)
        if parameters is not None:
            pulumi.set(__self__, "parameters", parameters)

    @_builtins.property
    @pulumi.getter
    def driver(self) -> Optional[_builtins.str]:
        """
        Driver is used to determine which kubelet plugin needs to be passed these configuration parameters.

        An admission policy provided by the driver developer could use this to decide whether it needs to validate them.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def parameters(self) -> Optional[Any]:
        """
        Parameters can contain arbitrary data. It is the responsibility of the driver developer to handle validation and versioning. Typically this includes self-identification and a version ("kind" + "apiVersion" for Kubernetes types), with conversion between different versions.

        The length of the raw data must be smaller or equal to 10 Ki.
        """
        return pulumi.get(self, "parameters")


@pulumi.output_type
class ResourceClaim(dict):
    """
    ResourceClaim describes a request for access to resources in the cluster, for use by workloads. For example, if a workload needs an accelerator device with specific properties, this is how that request is expressed. The status stanza tracks whether this claim has been satisfied and what specific resources have been allocated.

    This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiVersion":
            suggest = "api_version"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaim. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaim.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaim.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 spec: 'outputs.ResourceClaimSpec',
                 api_version: Optional[_builtins.str] = None,
                 kind: Optional[_builtins.str] = None,
                 metadata: Optional['_meta.v1.outputs.ObjectMeta'] = None,
                 status: Optional['outputs.ResourceClaimStatus'] = None):
        """
        ResourceClaim describes a request for access to resources in the cluster, for use by workloads. For example, if a workload needs an accelerator device with specific properties, this is how that request is expressed. The status stanza tracks whether this claim has been satisfied and what specific resources have been allocated.

        This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
        :param 'ResourceClaimSpecArgs' spec: Spec describes what is being requested and how to configure it. The spec is immutable.
        :param _builtins.str api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param _builtins.str kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param '_meta.v1.ObjectMetaArgs' metadata: Standard object metadata
        :param 'ResourceClaimStatusArgs' status: Status describes whether the claim is ready to use and what has been allocated.
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'resource.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'ResourceClaim')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> 'outputs.ResourceClaimSpec':
        """
        Spec describes what is being requested and how to configure it. The spec is immutable.
        """
        return pulumi.get(self, "spec")

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[_builtins.str]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[_builtins.str]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMeta']:
        """
        Standard object metadata
        """
        return pulumi.get(self, "metadata")

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional['outputs.ResourceClaimStatus']:
        """
        Status describes whether the claim is ready to use and what has been allocated.
        """
        return pulumi.get(self, "status")


@pulumi.output_type
class ResourceClaimConsumerReference(dict):
    """
    ResourceClaimConsumerReference contains enough information to let you locate the consumer of a ResourceClaim. The user must be a resource in the same namespace as the ResourceClaim.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiGroup":
            suggest = "api_group"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaimConsumerReference. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaimConsumerReference.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaimConsumerReference.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 name: _builtins.str,
                 resource: _builtins.str,
                 uid: _builtins.str,
                 api_group: Optional[_builtins.str] = None):
        """
        ResourceClaimConsumerReference contains enough information to let you locate the consumer of a ResourceClaim. The user must be a resource in the same namespace as the ResourceClaim.
        :param _builtins.str name: Name is the name of resource being referenced.
        :param _builtins.str resource: Resource is the type of resource being referenced, for example "pods".
        :param _builtins.str uid: UID identifies exactly one incarnation of the resource.
        :param _builtins.str api_group: APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "resource", resource)
        pulumi.set(__self__, "uid", uid)
        if api_group is not None:
            pulumi.set(__self__, "api_group", api_group)

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name is the name of resource being referenced.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def resource(self) -> _builtins.str:
        """
        Resource is the type of resource being referenced, for example "pods".
        """
        return pulumi.get(self, "resource")

    @_builtins.property
    @pulumi.getter
    def uid(self) -> _builtins.str:
        """
        UID identifies exactly one incarnation of the resource.
        """
        return pulumi.get(self, "uid")

    @_builtins.property
    @pulumi.getter(name="apiGroup")
    def api_group(self) -> Optional[_builtins.str]:
        """
        APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
        """
        return pulumi.get(self, "api_group")


@pulumi.output_type
class ResourceClaimConsumerReferencePatch(dict):
    """
    ResourceClaimConsumerReference contains enough information to let you locate the consumer of a ResourceClaim. The user must be a resource in the same namespace as the ResourceClaim.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiGroup":
            suggest = "api_group"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaimConsumerReferencePatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaimConsumerReferencePatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaimConsumerReferencePatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 api_group: Optional[_builtins.str] = None,
                 name: Optional[_builtins.str] = None,
                 resource: Optional[_builtins.str] = None,
                 uid: Optional[_builtins.str] = None):
        """
        ResourceClaimConsumerReference contains enough information to let you locate the consumer of a ResourceClaim. The user must be a resource in the same namespace as the ResourceClaim.
        :param _builtins.str api_group: APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
        :param _builtins.str name: Name is the name of resource being referenced.
        :param _builtins.str resource: Resource is the type of resource being referenced, for example "pods".
        :param _builtins.str uid: UID identifies exactly one incarnation of the resource.
        """
        if api_group is not None:
            pulumi.set(__self__, "api_group", api_group)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if resource is not None:
            pulumi.set(__self__, "resource", resource)
        if uid is not None:
            pulumi.set(__self__, "uid", uid)

    @_builtins.property
    @pulumi.getter(name="apiGroup")
    def api_group(self) -> Optional[_builtins.str]:
        """
        APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.
        """
        return pulumi.get(self, "api_group")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name is the name of resource being referenced.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def resource(self) -> Optional[_builtins.str]:
        """
        Resource is the type of resource being referenced, for example "pods".
        """
        return pulumi.get(self, "resource")

    @_builtins.property
    @pulumi.getter
    def uid(self) -> Optional[_builtins.str]:
        """
        UID identifies exactly one incarnation of the resource.
        """
        return pulumi.get(self, "uid")


@pulumi.output_type
class ResourceClaimSpec(dict):
    """
    ResourceClaimSpec defines what is being requested in a ResourceClaim and how to configure it.
    """
    def __init__(__self__, *,
                 devices: Optional['outputs.DeviceClaim'] = None):
        """
        ResourceClaimSpec defines what is being requested in a ResourceClaim and how to configure it.
        :param 'DeviceClaimArgs' devices: Devices defines how to request devices.
        """
        if devices is not None:
            pulumi.set(__self__, "devices", devices)

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional['outputs.DeviceClaim']:
        """
        Devices defines how to request devices.
        """
        return pulumi.get(self, "devices")


@pulumi.output_type
class ResourceClaimSpecPatch(dict):
    """
    ResourceClaimSpec defines what is being requested in a ResourceClaim and how to configure it.
    """
    def __init__(__self__, *,
                 devices: Optional['outputs.DeviceClaimPatch'] = None):
        """
        ResourceClaimSpec defines what is being requested in a ResourceClaim and how to configure it.
        :param 'DeviceClaimPatchArgs' devices: Devices defines how to request devices.
        """
        if devices is not None:
            pulumi.set(__self__, "devices", devices)

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional['outputs.DeviceClaimPatch']:
        """
        Devices defines how to request devices.
        """
        return pulumi.get(self, "devices")


@pulumi.output_type
class ResourceClaimStatus(dict):
    """
    ResourceClaimStatus tracks whether the resource has been allocated and what the result of that was.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "reservedFor":
            suggest = "reserved_for"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaimStatus. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaimStatus.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaimStatus.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 allocation: Optional['outputs.AllocationResult'] = None,
                 devices: Optional[Sequence['outputs.AllocatedDeviceStatus']] = None,
                 reserved_for: Optional[Sequence['outputs.ResourceClaimConsumerReference']] = None):
        """
        ResourceClaimStatus tracks whether the resource has been allocated and what the result of that was.
        :param 'AllocationResultArgs' allocation: Allocation is set once the claim has been allocated successfully.
        :param Sequence['AllocatedDeviceStatusArgs'] devices: Devices contains the status of each device allocated for this claim, as reported by the driver. This can include driver-specific information. Entries are owned by their respective drivers.
        :param Sequence['ResourceClaimConsumerReferenceArgs'] reserved_for: ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started. A claim that is in use or might be in use because it has been reserved must not get deallocated.
               
               In a cluster with multiple scheduler instances, two pods might get scheduled concurrently by different schedulers. When they reference the same ResourceClaim which already has reached its maximum number of consumers, only one pod can be scheduled.
               
               Both schedulers try to add their pod to the claim.status.reservedFor field, but only the update that reaches the API server first gets stored. The other one fails with an error and the scheduler which issued it knows that it must put the pod back into the queue, waiting for the ResourceClaim to become usable again.
               
               There can be at most 256 such reservations. This may get increased in the future, but not reduced.
        """
        if allocation is not None:
            pulumi.set(__self__, "allocation", allocation)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if reserved_for is not None:
            pulumi.set(__self__, "reserved_for", reserved_for)

    @_builtins.property
    @pulumi.getter
    def allocation(self) -> Optional['outputs.AllocationResult']:
        """
        Allocation is set once the claim has been allocated successfully.
        """
        return pulumi.get(self, "allocation")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional[Sequence['outputs.AllocatedDeviceStatus']]:
        """
        Devices contains the status of each device allocated for this claim, as reported by the driver. This can include driver-specific information. Entries are owned by their respective drivers.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter(name="reservedFor")
    def reserved_for(self) -> Optional[Sequence['outputs.ResourceClaimConsumerReference']]:
        """
        ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started. A claim that is in use or might be in use because it has been reserved must not get deallocated.

        In a cluster with multiple scheduler instances, two pods might get scheduled concurrently by different schedulers. When they reference the same ResourceClaim which already has reached its maximum number of consumers, only one pod can be scheduled.

        Both schedulers try to add their pod to the claim.status.reservedFor field, but only the update that reaches the API server first gets stored. The other one fails with an error and the scheduler which issued it knows that it must put the pod back into the queue, waiting for the ResourceClaim to become usable again.

        There can be at most 256 such reservations. This may get increased in the future, but not reduced.
        """
        return pulumi.get(self, "reserved_for")


@pulumi.output_type
class ResourceClaimStatusPatch(dict):
    """
    ResourceClaimStatus tracks whether the resource has been allocated and what the result of that was.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "reservedFor":
            suggest = "reserved_for"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaimStatusPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaimStatusPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaimStatusPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 allocation: Optional['outputs.AllocationResultPatch'] = None,
                 devices: Optional[Sequence['outputs.AllocatedDeviceStatusPatch']] = None,
                 reserved_for: Optional[Sequence['outputs.ResourceClaimConsumerReferencePatch']] = None):
        """
        ResourceClaimStatus tracks whether the resource has been allocated and what the result of that was.
        :param 'AllocationResultPatchArgs' allocation: Allocation is set once the claim has been allocated successfully.
        :param Sequence['AllocatedDeviceStatusPatchArgs'] devices: Devices contains the status of each device allocated for this claim, as reported by the driver. This can include driver-specific information. Entries are owned by their respective drivers.
        :param Sequence['ResourceClaimConsumerReferencePatchArgs'] reserved_for: ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started. A claim that is in use or might be in use because it has been reserved must not get deallocated.
               
               In a cluster with multiple scheduler instances, two pods might get scheduled concurrently by different schedulers. When they reference the same ResourceClaim which already has reached its maximum number of consumers, only one pod can be scheduled.
               
               Both schedulers try to add their pod to the claim.status.reservedFor field, but only the update that reaches the API server first gets stored. The other one fails with an error and the scheduler which issued it knows that it must put the pod back into the queue, waiting for the ResourceClaim to become usable again.
               
               There can be at most 256 such reservations. This may get increased in the future, but not reduced.
        """
        if allocation is not None:
            pulumi.set(__self__, "allocation", allocation)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if reserved_for is not None:
            pulumi.set(__self__, "reserved_for", reserved_for)

    @_builtins.property
    @pulumi.getter
    def allocation(self) -> Optional['outputs.AllocationResultPatch']:
        """
        Allocation is set once the claim has been allocated successfully.
        """
        return pulumi.get(self, "allocation")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional[Sequence['outputs.AllocatedDeviceStatusPatch']]:
        """
        Devices contains the status of each device allocated for this claim, as reported by the driver. This can include driver-specific information. Entries are owned by their respective drivers.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter(name="reservedFor")
    def reserved_for(self) -> Optional[Sequence['outputs.ResourceClaimConsumerReferencePatch']]:
        """
        ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started. A claim that is in use or might be in use because it has been reserved must not get deallocated.

        In a cluster with multiple scheduler instances, two pods might get scheduled concurrently by different schedulers. When they reference the same ResourceClaim which already has reached its maximum number of consumers, only one pod can be scheduled.

        Both schedulers try to add their pod to the claim.status.reservedFor field, but only the update that reaches the API server first gets stored. The other one fails with an error and the scheduler which issued it knows that it must put the pod back into the queue, waiting for the ResourceClaim to become usable again.

        There can be at most 256 such reservations. This may get increased in the future, but not reduced.
        """
        return pulumi.get(self, "reserved_for")


@pulumi.output_type
class ResourceClaimTemplate(dict):
    """
    ResourceClaimTemplate is used to produce ResourceClaim objects.

    This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiVersion":
            suggest = "api_version"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceClaimTemplate. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceClaimTemplate.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceClaimTemplate.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 spec: 'outputs.ResourceClaimTemplateSpec',
                 api_version: Optional[_builtins.str] = None,
                 kind: Optional[_builtins.str] = None,
                 metadata: Optional['_meta.v1.outputs.ObjectMeta'] = None):
        """
        ResourceClaimTemplate is used to produce ResourceClaim objects.

        This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
        :param 'ResourceClaimTemplateSpecArgs' spec: Describes the ResourceClaim that is to be generated.
               
               This field is immutable. A ResourceClaim will get created by the control plane for a Pod when needed and then not get updated anymore.
        :param _builtins.str api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param _builtins.str kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param '_meta.v1.ObjectMetaArgs' metadata: Standard object metadata
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'resource.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'ResourceClaimTemplate')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> 'outputs.ResourceClaimTemplateSpec':
        """
        Describes the ResourceClaim that is to be generated.

        This field is immutable. A ResourceClaim will get created by the control plane for a Pod when needed and then not get updated anymore.
        """
        return pulumi.get(self, "spec")

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[_builtins.str]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[_builtins.str]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMeta']:
        """
        Standard object metadata
        """
        return pulumi.get(self, "metadata")


@pulumi.output_type
class ResourceClaimTemplateSpec(dict):
    """
    ResourceClaimTemplateSpec contains the metadata and fields for a ResourceClaim.
    """
    def __init__(__self__, *,
                 spec: 'outputs.ResourceClaimSpec',
                 metadata: Optional['_meta.v1.outputs.ObjectMeta'] = None):
        """
        ResourceClaimTemplateSpec contains the metadata and fields for a ResourceClaim.
        :param 'ResourceClaimSpecArgs' spec: Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
        :param '_meta.v1.ObjectMetaArgs' metadata: ObjectMeta may contain labels and annotations that will be copied into the ResourceClaim when creating it. No other fields are allowed and will be rejected during validation.
        """
        pulumi.set(__self__, "spec", spec)
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> 'outputs.ResourceClaimSpec':
        """
        Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
        """
        return pulumi.get(self, "spec")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMeta']:
        """
        ObjectMeta may contain labels and annotations that will be copied into the ResourceClaim when creating it. No other fields are allowed and will be rejected during validation.
        """
        return pulumi.get(self, "metadata")


@pulumi.output_type
class ResourceClaimTemplateSpecPatch(dict):
    """
    ResourceClaimTemplateSpec contains the metadata and fields for a ResourceClaim.
    """
    def __init__(__self__, *,
                 metadata: Optional['_meta.v1.outputs.ObjectMetaPatch'] = None,
                 spec: Optional['outputs.ResourceClaimSpecPatch'] = None):
        """
        ResourceClaimTemplateSpec contains the metadata and fields for a ResourceClaim.
        :param '_meta.v1.ObjectMetaPatchArgs' metadata: ObjectMeta may contain labels and annotations that will be copied into the ResourceClaim when creating it. No other fields are allowed and will be rejected during validation.
        :param 'ResourceClaimSpecPatchArgs' spec: Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
        """
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)
        if spec is not None:
            pulumi.set(__self__, "spec", spec)

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMetaPatch']:
        """
        ObjectMeta may contain labels and annotations that will be copied into the ResourceClaim when creating it. No other fields are allowed and will be rejected during validation.
        """
        return pulumi.get(self, "metadata")

    @_builtins.property
    @pulumi.getter
    def spec(self) -> Optional['outputs.ResourceClaimSpecPatch']:
        """
        Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here.
        """
        return pulumi.get(self, "spec")


@pulumi.output_type
class ResourcePool(dict):
    """
    ResourcePool describes the pool that ResourceSlices belong to.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "resourceSliceCount":
            suggest = "resource_slice_count"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourcePool. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourcePool.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourcePool.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 generation: _builtins.int,
                 name: _builtins.str,
                 resource_slice_count: _builtins.int):
        """
        ResourcePool describes the pool that ResourceSlices belong to.
        :param _builtins.int generation: Generation tracks the change in a pool over time. Whenever a driver changes something about one or more of the resources in a pool, it must change the generation in all ResourceSlices which are part of that pool. Consumers of ResourceSlices should only consider resources from the pool with the highest generation number. The generation may be reset by drivers, which should be fine for consumers, assuming that all ResourceSlices in a pool are updated to match or deleted.
               
               Combined with ResourceSliceCount, this mechanism enables consumers to detect pools which are comprised of multiple ResourceSlices and are in an incomplete state.
        :param _builtins.str name: Name is used to identify the pool. For node-local devices, this is often the node name, but this is not required.
               
               It must not be longer than 253 characters and must consist of one or more DNS sub-domains separated by slashes. This field is immutable.
        :param _builtins.int resource_slice_count: ResourceSliceCount is the total number of ResourceSlices in the pool at this generation number. Must be greater than zero.
               
               Consumers can use this to check whether they have seen all ResourceSlices belonging to the same pool.
        """
        pulumi.set(__self__, "generation", generation)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "resource_slice_count", resource_slice_count)

    @_builtins.property
    @pulumi.getter
    def generation(self) -> _builtins.int:
        """
        Generation tracks the change in a pool over time. Whenever a driver changes something about one or more of the resources in a pool, it must change the generation in all ResourceSlices which are part of that pool. Consumers of ResourceSlices should only consider resources from the pool with the highest generation number. The generation may be reset by drivers, which should be fine for consumers, assuming that all ResourceSlices in a pool are updated to match or deleted.

        Combined with ResourceSliceCount, this mechanism enables consumers to detect pools which are comprised of multiple ResourceSlices and are in an incomplete state.
        """
        return pulumi.get(self, "generation")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        """
        Name is used to identify the pool. For node-local devices, this is often the node name, but this is not required.

        It must not be longer than 253 characters and must consist of one or more DNS sub-domains separated by slashes. This field is immutable.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="resourceSliceCount")
    def resource_slice_count(self) -> _builtins.int:
        """
        ResourceSliceCount is the total number of ResourceSlices in the pool at this generation number. Must be greater than zero.

        Consumers can use this to check whether they have seen all ResourceSlices belonging to the same pool.
        """
        return pulumi.get(self, "resource_slice_count")


@pulumi.output_type
class ResourcePoolPatch(dict):
    """
    ResourcePool describes the pool that ResourceSlices belong to.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "resourceSliceCount":
            suggest = "resource_slice_count"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourcePoolPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourcePoolPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourcePoolPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 generation: Optional[_builtins.int] = None,
                 name: Optional[_builtins.str] = None,
                 resource_slice_count: Optional[_builtins.int] = None):
        """
        ResourcePool describes the pool that ResourceSlices belong to.
        :param _builtins.int generation: Generation tracks the change in a pool over time. Whenever a driver changes something about one or more of the resources in a pool, it must change the generation in all ResourceSlices which are part of that pool. Consumers of ResourceSlices should only consider resources from the pool with the highest generation number. The generation may be reset by drivers, which should be fine for consumers, assuming that all ResourceSlices in a pool are updated to match or deleted.
               
               Combined with ResourceSliceCount, this mechanism enables consumers to detect pools which are comprised of multiple ResourceSlices and are in an incomplete state.
        :param _builtins.str name: Name is used to identify the pool. For node-local devices, this is often the node name, but this is not required.
               
               It must not be longer than 253 characters and must consist of one or more DNS sub-domains separated by slashes. This field is immutable.
        :param _builtins.int resource_slice_count: ResourceSliceCount is the total number of ResourceSlices in the pool at this generation number. Must be greater than zero.
               
               Consumers can use this to check whether they have seen all ResourceSlices belonging to the same pool.
        """
        if generation is not None:
            pulumi.set(__self__, "generation", generation)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if resource_slice_count is not None:
            pulumi.set(__self__, "resource_slice_count", resource_slice_count)

    @_builtins.property
    @pulumi.getter
    def generation(self) -> Optional[_builtins.int]:
        """
        Generation tracks the change in a pool over time. Whenever a driver changes something about one or more of the resources in a pool, it must change the generation in all ResourceSlices which are part of that pool. Consumers of ResourceSlices should only consider resources from the pool with the highest generation number. The generation may be reset by drivers, which should be fine for consumers, assuming that all ResourceSlices in a pool are updated to match or deleted.

        Combined with ResourceSliceCount, this mechanism enables consumers to detect pools which are comprised of multiple ResourceSlices and are in an incomplete state.
        """
        return pulumi.get(self, "generation")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        """
        Name is used to identify the pool. For node-local devices, this is often the node name, but this is not required.

        It must not be longer than 253 characters and must consist of one or more DNS sub-domains separated by slashes. This field is immutable.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="resourceSliceCount")
    def resource_slice_count(self) -> Optional[_builtins.int]:
        """
        ResourceSliceCount is the total number of ResourceSlices in the pool at this generation number. Must be greater than zero.

        Consumers can use this to check whether they have seen all ResourceSlices belonging to the same pool.
        """
        return pulumi.get(self, "resource_slice_count")


@pulumi.output_type
class ResourceSlice(dict):
    """
    ResourceSlice represents one or more resources in a pool of similar resources, managed by a common driver. A pool may span more than one ResourceSlice, and exactly how many ResourceSlices comprise a pool is determined by the driver.

    At the moment, the only supported resources are devices with attributes and capacities. Each device in a given pool, regardless of how many ResourceSlices, must have a unique name. The ResourceSlice in which a device gets published may change over time. The unique identifier for a device is the tuple <driver name>, <pool name>, <device name>.

    Whenever a driver needs to update a pool, it increments the pool.Spec.Pool.Generation number and updates all ResourceSlices with that new number and new resource definitions. A consumer must only use ResourceSlices with the highest generation number and ignore all others.

    When allocating all resources in a pool matching certain criteria or when looking for the best solution among several different alternatives, a consumer should check the number of ResourceSlices in a pool (included in each ResourceSlice) to determine whether its view of a pool is complete and if not, should wait until the driver has completed updating the pool.

    For resources that are not local to a node, the node name is not set. Instead, the driver may use a node selector to specify where the devices are available.

    This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "apiVersion":
            suggest = "api_version"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceSlice. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceSlice.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceSlice.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 spec: 'outputs.ResourceSliceSpec',
                 api_version: Optional[_builtins.str] = None,
                 kind: Optional[_builtins.str] = None,
                 metadata: Optional['_meta.v1.outputs.ObjectMeta'] = None):
        """
        ResourceSlice represents one or more resources in a pool of similar resources, managed by a common driver. A pool may span more than one ResourceSlice, and exactly how many ResourceSlices comprise a pool is determined by the driver.

        At the moment, the only supported resources are devices with attributes and capacities. Each device in a given pool, regardless of how many ResourceSlices, must have a unique name. The ResourceSlice in which a device gets published may change over time. The unique identifier for a device is the tuple <driver name>, <pool name>, <device name>.

        Whenever a driver needs to update a pool, it increments the pool.Spec.Pool.Generation number and updates all ResourceSlices with that new number and new resource definitions. A consumer must only use ResourceSlices with the highest generation number and ignore all others.

        When allocating all resources in a pool matching certain criteria or when looking for the best solution among several different alternatives, a consumer should check the number of ResourceSlices in a pool (included in each ResourceSlice) to determine whether its view of a pool is complete and if not, should wait until the driver has completed updating the pool.

        For resources that are not local to a node, the node name is not set. Instead, the driver may use a node selector to specify where the devices are available.

        This is an alpha type and requires enabling the DynamicResourceAllocation feature gate.
        :param 'ResourceSliceSpecArgs' spec: Contains the information published by the driver.
               
               Changing the spec automatically increments the metadata.generation number.
        :param _builtins.str api_version: APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        :param _builtins.str kind: Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        :param '_meta.v1.ObjectMetaArgs' metadata: Standard object metadata
        """
        pulumi.set(__self__, "spec", spec)
        if api_version is not None:
            pulumi.set(__self__, "api_version", 'resource.k8s.io/v1beta1')
        if kind is not None:
            pulumi.set(__self__, "kind", 'ResourceSlice')
        if metadata is not None:
            pulumi.set(__self__, "metadata", metadata)

    @_builtins.property
    @pulumi.getter
    def spec(self) -> 'outputs.ResourceSliceSpec':
        """
        Contains the information published by the driver.

        Changing the spec automatically increments the metadata.generation number.
        """
        return pulumi.get(self, "spec")

    @_builtins.property
    @pulumi.getter(name="apiVersion")
    def api_version(self) -> Optional[_builtins.str]:
        """
        APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
        """
        return pulumi.get(self, "api_version")

    @_builtins.property
    @pulumi.getter
    def kind(self) -> Optional[_builtins.str]:
        """
        Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
        """
        return pulumi.get(self, "kind")

    @_builtins.property
    @pulumi.getter
    def metadata(self) -> Optional['_meta.v1.outputs.ObjectMeta']:
        """
        Standard object metadata
        """
        return pulumi.get(self, "metadata")


@pulumi.output_type
class ResourceSliceSpec(dict):
    """
    ResourceSliceSpec contains the information published by the driver in one ResourceSlice.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allNodes":
            suggest = "all_nodes"
        elif key == "nodeName":
            suggest = "node_name"
        elif key == "nodeSelector":
            suggest = "node_selector"
        elif key == "perDeviceNodeSelection":
            suggest = "per_device_node_selection"
        elif key == "sharedCounters":
            suggest = "shared_counters"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceSliceSpec. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceSliceSpec.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceSliceSpec.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 driver: _builtins.str,
                 pool: 'outputs.ResourcePool',
                 all_nodes: Optional[_builtins.bool] = None,
                 devices: Optional[Sequence['outputs.Device']] = None,
                 node_name: Optional[_builtins.str] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelector'] = None,
                 per_device_node_selection: Optional[_builtins.bool] = None,
                 shared_counters: Optional[Sequence['outputs.CounterSet']] = None):
        """
        ResourceSliceSpec contains the information published by the driver in one ResourceSlice.
        :param _builtins.str driver: Driver identifies the DRA driver providing the capacity information. A field selector can be used to list only ResourceSlice objects with a certain driver name.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters. This field is immutable.
        :param 'ResourcePoolArgs' pool: Pool describes the pool that this ResourceSlice belongs to.
        :param _builtins.bool all_nodes: AllNodes indicates that all nodes have access to the resources in the pool.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param Sequence['DeviceArgs'] devices: Devices lists some or all of the devices in this pool.
               
               Must not have more than 128 entries. If any device uses taints or consumes counters the limit is 64.
               
               Only one of Devices and SharedCounters can be set in a ResourceSlice.
        :param _builtins.str node_name: NodeName identifies the node which provides the resources in this pool. A field selector can be used to list only ResourceSlice objects belonging to a certain node.
               
               This field can be used to limit access from nodes to ResourceSlices with the same node name. It also indicates to autoscalers that adding new nodes of the same type as some old node might also make new resources available.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set. This field is immutable.
        :param '_core.v1.NodeSelectorArgs' node_selector: NodeSelector defines which nodes have access to the resources in the pool, when that pool is not limited to a single node.
               
               Must use exactly one term.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param _builtins.bool per_device_node_selection: PerDeviceNodeSelection defines whether the access from nodes to resources in the pool is set on the ResourceSlice level or on each device. If it is set to true, every device defined the ResourceSlice must specify this individually.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param Sequence['CounterSetArgs'] shared_counters: SharedCounters defines a list of counter sets, each of which has a name and a list of counters available.
               
               The names of the counter sets must be unique in the ResourcePool.
               
               Only one of Devices and SharedCounters can be set in a ResourceSlice.
               
               The maximum number of counter sets is 8.
        """
        pulumi.set(__self__, "driver", driver)
        pulumi.set(__self__, "pool", pool)
        if all_nodes is not None:
            pulumi.set(__self__, "all_nodes", all_nodes)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)
        if per_device_node_selection is not None:
            pulumi.set(__self__, "per_device_node_selection", per_device_node_selection)
        if shared_counters is not None:
            pulumi.set(__self__, "shared_counters", shared_counters)

    @_builtins.property
    @pulumi.getter
    def driver(self) -> _builtins.str:
        """
        Driver identifies the DRA driver providing the capacity information. A field selector can be used to list only ResourceSlice objects with a certain driver name.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters. This field is immutable.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> 'outputs.ResourcePool':
        """
        Pool describes the pool that this ResourceSlice belongs to.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter(name="allNodes")
    def all_nodes(self) -> Optional[_builtins.bool]:
        """
        AllNodes indicates that all nodes have access to the resources in the pool.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "all_nodes")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional[Sequence['outputs.Device']]:
        """
        Devices lists some or all of the devices in this pool.

        Must not have more than 128 entries. If any device uses taints or consumes counters the limit is 64.

        Only one of Devices and SharedCounters can be set in a ResourceSlice.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[_builtins.str]:
        """
        NodeName identifies the node which provides the resources in this pool. A field selector can be used to list only ResourceSlice objects belonging to a certain node.

        This field can be used to limit access from nodes to ResourceSlices with the same node name. It also indicates to autoscalers that adding new nodes of the same type as some old node might also make new resources available.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set. This field is immutable.
        """
        return pulumi.get(self, "node_name")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelector']:
        """
        NodeSelector defines which nodes have access to the resources in the pool, when that pool is not limited to a single node.

        Must use exactly one term.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "node_selector")

    @_builtins.property
    @pulumi.getter(name="perDeviceNodeSelection")
    def per_device_node_selection(self) -> Optional[_builtins.bool]:
        """
        PerDeviceNodeSelection defines whether the access from nodes to resources in the pool is set on the ResourceSlice level or on each device. If it is set to true, every device defined the ResourceSlice must specify this individually.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "per_device_node_selection")

    @_builtins.property
    @pulumi.getter(name="sharedCounters")
    def shared_counters(self) -> Optional[Sequence['outputs.CounterSet']]:
        """
        SharedCounters defines a list of counter sets, each of which has a name and a list of counters available.

        The names of the counter sets must be unique in the ResourcePool.

        Only one of Devices and SharedCounters can be set in a ResourceSlice.

        The maximum number of counter sets is 8.
        """
        return pulumi.get(self, "shared_counters")


@pulumi.output_type
class ResourceSliceSpecPatch(dict):
    """
    ResourceSliceSpec contains the information published by the driver in one ResourceSlice.
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "allNodes":
            suggest = "all_nodes"
        elif key == "nodeName":
            suggest = "node_name"
        elif key == "nodeSelector":
            suggest = "node_selector"
        elif key == "perDeviceNodeSelection":
            suggest = "per_device_node_selection"
        elif key == "sharedCounters":
            suggest = "shared_counters"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ResourceSliceSpecPatch. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ResourceSliceSpecPatch.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ResourceSliceSpecPatch.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 all_nodes: Optional[_builtins.bool] = None,
                 devices: Optional[Sequence['outputs.DevicePatch']] = None,
                 driver: Optional[_builtins.str] = None,
                 node_name: Optional[_builtins.str] = None,
                 node_selector: Optional['_core.v1.outputs.NodeSelectorPatch'] = None,
                 per_device_node_selection: Optional[_builtins.bool] = None,
                 pool: Optional['outputs.ResourcePoolPatch'] = None,
                 shared_counters: Optional[Sequence['outputs.CounterSetPatch']] = None):
        """
        ResourceSliceSpec contains the information published by the driver in one ResourceSlice.
        :param _builtins.bool all_nodes: AllNodes indicates that all nodes have access to the resources in the pool.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param Sequence['DevicePatchArgs'] devices: Devices lists some or all of the devices in this pool.
               
               Must not have more than 128 entries. If any device uses taints or consumes counters the limit is 64.
               
               Only one of Devices and SharedCounters can be set in a ResourceSlice.
        :param _builtins.str driver: Driver identifies the DRA driver providing the capacity information. A field selector can be used to list only ResourceSlice objects with a certain driver name.
               
               Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters. This field is immutable.
        :param _builtins.str node_name: NodeName identifies the node which provides the resources in this pool. A field selector can be used to list only ResourceSlice objects belonging to a certain node.
               
               This field can be used to limit access from nodes to ResourceSlices with the same node name. It also indicates to autoscalers that adding new nodes of the same type as some old node might also make new resources available.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set. This field is immutable.
        :param '_core.v1.NodeSelectorPatchArgs' node_selector: NodeSelector defines which nodes have access to the resources in the pool, when that pool is not limited to a single node.
               
               Must use exactly one term.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param _builtins.bool per_device_node_selection: PerDeviceNodeSelection defines whether the access from nodes to resources in the pool is set on the ResourceSlice level or on each device. If it is set to true, every device defined the ResourceSlice must specify this individually.
               
               Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        :param 'ResourcePoolPatchArgs' pool: Pool describes the pool that this ResourceSlice belongs to.
        :param Sequence['CounterSetPatchArgs'] shared_counters: SharedCounters defines a list of counter sets, each of which has a name and a list of counters available.
               
               The names of the counter sets must be unique in the ResourcePool.
               
               Only one of Devices and SharedCounters can be set in a ResourceSlice.
               
               The maximum number of counter sets is 8.
        """
        if all_nodes is not None:
            pulumi.set(__self__, "all_nodes", all_nodes)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if driver is not None:
            pulumi.set(__self__, "driver", driver)
        if node_name is not None:
            pulumi.set(__self__, "node_name", node_name)
        if node_selector is not None:
            pulumi.set(__self__, "node_selector", node_selector)
        if per_device_node_selection is not None:
            pulumi.set(__self__, "per_device_node_selection", per_device_node_selection)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if shared_counters is not None:
            pulumi.set(__self__, "shared_counters", shared_counters)

    @_builtins.property
    @pulumi.getter(name="allNodes")
    def all_nodes(self) -> Optional[_builtins.bool]:
        """
        AllNodes indicates that all nodes have access to the resources in the pool.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "all_nodes")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional[Sequence['outputs.DevicePatch']]:
        """
        Devices lists some or all of the devices in this pool.

        Must not have more than 128 entries. If any device uses taints or consumes counters the limit is 64.

        Only one of Devices and SharedCounters can be set in a ResourceSlice.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter
    def driver(self) -> Optional[_builtins.str]:
        """
        Driver identifies the DRA driver providing the capacity information. A field selector can be used to list only ResourceSlice objects with a certain driver name.

        Must be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. It should use only lower case characters. This field is immutable.
        """
        return pulumi.get(self, "driver")

    @_builtins.property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[_builtins.str]:
        """
        NodeName identifies the node which provides the resources in this pool. A field selector can be used to list only ResourceSlice objects belonging to a certain node.

        This field can be used to limit access from nodes to ResourceSlices with the same node name. It also indicates to autoscalers that adding new nodes of the same type as some old node might also make new resources available.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set. This field is immutable.
        """
        return pulumi.get(self, "node_name")

    @_builtins.property
    @pulumi.getter(name="nodeSelector")
    def node_selector(self) -> Optional['_core.v1.outputs.NodeSelectorPatch']:
        """
        NodeSelector defines which nodes have access to the resources in the pool, when that pool is not limited to a single node.

        Must use exactly one term.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "node_selector")

    @_builtins.property
    @pulumi.getter(name="perDeviceNodeSelection")
    def per_device_node_selection(self) -> Optional[_builtins.bool]:
        """
        PerDeviceNodeSelection defines whether the access from nodes to resources in the pool is set on the ResourceSlice level or on each device. If it is set to true, every device defined the ResourceSlice must specify this individually.

        Exactly one of NodeName, NodeSelector, AllNodes, and PerDeviceNodeSelection must be set.
        """
        return pulumi.get(self, "per_device_node_selection")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> Optional['outputs.ResourcePoolPatch']:
        """
        Pool describes the pool that this ResourceSlice belongs to.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter(name="sharedCounters")
    def shared_counters(self) -> Optional[Sequence['outputs.CounterSetPatch']]:
        """
        SharedCounters defines a list of counter sets, each of which has a name and a list of counters available.

        The names of the counter sets must be unique in the ResourcePool.

        Only one of Devices and SharedCounters can be set in a ResourceSlice.

        The maximum number of counter sets is 8.
        """
        return pulumi.get(self, "shared_counters")


